/*
 * IOread.c - read input etc 
 */

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include "header.h"

/* 
 * Read the command line 
 */
read_commandline(argc,argv) 
int argc;
char **argv;
{ 
   FILE   *optionfile, *fopen();
   extern char   xlabel[MAXCHAR];
   extern char   ylabel[MAXCHAR];
   extern char   toplabel[MAXCHAR];
   extern char   printer[MAXCHAR];
   extern int    grid, equalscale, landscape, clip;
   extern int    postscript, printplot, noplot;
   extern int    xticks,yticks,xlog,ylog,zlog;
   extern int    linetypes, contlabel, joinlevel;
   extern int    expand_mesh, expand_cont, print_slice;
   extern double scale, xyratio;
   extern double cont_level;
   extern double cstep;
   extern int    nsteps;
   int    i,FILEFOUND;

   FILEFOUND = FALSE;
   for (i=1; i < argc; i++) {                     /* Parse Line */
      if (argv[i][0] == '=') {
         geometry = argv[i];
         continue;
         }
      if (strncmp(argv[i],"-geom",5)==0) {        /* new geometry */
         if (++i >= argc) usage(progname);
         geometry = argv[i];
         continue;
         }
      if (index(argv[i],':')!=NULL) {             /* host:display */
         strncpy(display_name,argv[i],sizeof(display_name));
         continue;
         }
      if (strncmp(argv[i],"-d",2) == 0) {         /* new display */
         if (++i >= argc) usage(progname);
         display_name = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-rv") == 0) {           /* reverse display */
         reverse = 1;
         continue;
         }
      if (strcmp(argv[i],"-bw") == 0) {           /* border width */
         if (++i >= argc) usage(progname);
         border_width = atoi(argv[i]);
         continue;
         }
      if (strcmp(argv[i],"-bd") == 0) {           /* border color */
         if (++i >= argc) usage(progname);
         border_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-fg") == 0) {           /* foreground color */
         if (++i >= argc) usage(progname);
         fore_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-bg") == 0) {           /* background color */
         if (++i >= argc) usage(progname);
         back_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-fn") == 0) {           /* font type */
         if (++i >= argc) usage(progname);
         font_name  = argv[i];
         continue;
         }

      if ( (strcmp(argv[i],"-xl") == 0) ||        /* xlabel */
           (strcmp(argv[i],"-xlabel") == 0) ){
         if (++i >= argc) usage(progname);
         strcpy(xlabel,argv[i]);
         fprintf(stdout,"   Commands: xlabel          = \"%s\"\n",xlabel);
         continue;
         }
      if ( (strcmp(argv[i],"-yl") == 0) ||        /* ylabel */
           (strcmp(argv[i],"-ylabel") == 0) ){
         if (++i >= argc) usage(progname);
         strcpy(ylabel,argv[i]);
         fprintf(stdout,"   Commands: ylabel          = \"%s\"\n",ylabel);
         continue;
         }
      if ( (strcmp(argv[i],"-tl") == 0) ||        /* toplabel */
           (strcmp(argv[i],"-toplabel") == 0) ){
         if (++i >= argc) usage(progname);
         strcpy(toplabel,argv[i]);
         fprintf(stdout,"   Commands: toplabel        = \"%s\"\n",toplabel);
         continue;
         }
      if (argv[i][0]=='-' && argv[i][1]=='P') {   /* Printer */
         strcpy(printer,argv[i]); 
         fprintf(stdout,"   Commands: Printer         = %s\n",printer);
         continue;
         }
      if (strcmp(argv[i],"-e") == 0) {            /* equalscale */
         equalscale = OFF;
         fprintf(stdout,"   Commands: equalscale      = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-g") == 0) {            /* grid       */
         grid = ON;
         fprintf(stdout,"   Commands: grid            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-l") == 0) {            /* landscape  */
         landscape = ON;
         fprintf(stdout,"   Commands: landscape       = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-clip") == 0) {         /* clip       */
         clip      = OFF;
         fprintf(stdout,"   Commands: clip            = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-ps") == 0) {           /* postscript */
         postscript = OFF;
         fprintf(stdout,"   Commands: postscript      = OFF\n");
         continue;
         }
      if ( (strcmp(argv[i],"-print") == 0) ||
           (strcmp(argv[i],"-prt")   == 0) ) {    /* print      */
         printplot  = ON;
         fprintf(stdout,"   Commands: printplot       = ON\n");
         continue;
         }
      if (strncmp(argv[i],"-nop",3) == 0) {       /* noplot     */
         noplot = ON;
         fprintf(stdout,"   Commands: noplot          = ON\n");
         continue;
         }
      if (strncmp(argv[i],"-xt",3) == 0) {        /* xticks     */
         if (++i >= argc) usage(progname);
         xticks = atoi(argv[i]);
         if (xticks <= 0 || xticks > 20) {
            xticks = 4;
            fprintf(stdout,"   Warning : Invalid no of xticks -");
            fprintf(stdout," xticks reset to %d\n",xticks);
         }
         fprintf(stdout,"   Commands: xticks          = %d\n",xticks);
         continue;
         }
      if (strncmp(argv[i],"-yt",3) == 0) {        /* yticks     */
         if (++i >= argc) usage(progname);
         yticks = atoi(argv[i]);
         if (yticks <= 0 || yticks > 20) {
            yticks = 4;
            fprintf(stdout,"   Warning : Invalid no of yticks -");
            fprintf(stdout," yticks reset to %d\n",yticks);
         }
         fprintf(stdout,"   Commands: yticks          = %d\n",yticks);
         continue;
         }
      if (strcmp(argv[i],"-xlog") == 0) {         /* xlog       */
         xlog   = ON;
         fprintf(stdout,"   Commands: xlog            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-ylog") == 0) {         /* ylog       */
         ylog   = ON;
         fprintf(stdout,"   Commands: ylog            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-xar") == 0) {          /* xautorange */
         xautorange = OFF;
         fprintf(stdout,"   Commands: xautorange      = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-yar") == 0) {          /* yautorange */
         yautorange = OFF;
         fprintf(stdout,"   Commands: yautorange      = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-ar") == 0) {           /* autorange  */
         autorange  = OFF;
         xautorange = OFF;
         yautorange = OFF;
         fprintf(stdout,"   Commands: autorange       = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-s") == 0) {            /* scale factor */  
         if (++i >= argc) usage(progname);
         scale = atof(argv[i]);
         if (scale < 0.1 || scale > 1) {
            scale = 1.0;
            fprintf(stdout,"   Warning : Invalid scale -");
            fprintf(stdout," scale reset to %f\n",scale);
         }
         fprintf(stdout,"   Commands: scale           = %f\n",scale);
         continue;
         }
      if (strcmp(argv[i],"-xyratio") == 0) {      /* axis ratio */  
         if (++i >= argc) usage(progname);
         xyratio = atof(argv[i]);
         if (xyratio < 0.01 || xyratio > 100.0) {
            xyratio = 0.75;
            fprintf(stdout,"   Warning : Invalid xyratio -");
            fprintf(stdout," xyratio reset to %f\n",xyratio);
         }
         equalscale = OFF;
         fprintf(stdout,"   Commands: xyratio         = %f\n",xyratio);
         continue;
         }

      if (strcmp(argv[i],"-cstep") == 0) {        /* contour step size */  
         if (++i >= argc) usage(progname);
         cstep = atof(argv[i]);
         if (cstep > 1.0e-5) nsteps = 0;
         fprintf(stdout,"   Commands: cstep           = %f\n",cstep);
         continue;
         }
      if (strcmp(argv[i],"-nsteps") == 0) {       /* no of contour steps */  
         if (++i >= argc) usage(progname);
         nsteps = atoi(argv[i]);
         if (nsteps > 0) cstep = 0.0;
         fprintf(stdout,"   Commands: nsteps          = %f\n",nsteps);
         continue;
         }
      if (strcmp(argv[i],"-exp") == 0) {          /* expand into 3D */
         expand_mesh = ON;
         expand_cont = ON;
         fprintf(stdout,"   Commands: expand_mesh (3D)     = ON\n");
         fprintf(stdout,"   Commands: expand_cont (3D)     = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-expand_mesh") == 0) {  /* expand into 3D */
         expand_mesh = ON;
         fprintf(stdout,"   Commands: expand_mesh (3D)     = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-expand_cont") == 0) {  /* expand into 3D */
         expand_cont = ON;
         fprintf(stdout,"   Commands: expand_cont (3D)     = ON\n");
         continue;
         }
      if (strncmp(argv[i],"-j",2) == 0) {         /* join curves */
         if (++i >= argc) usage(progname);
         CNstring_to_upper(argv[i]);
         if      (strcmp(argv[i],"HIGH")==0) joinlevel = CN_HIGH;
         else if (strcmp(argv[i],"LOW" )==0) joinlevel = CN_LOW;
         else {
            fprintf(stdout,"   Warning : Invalid keyword -");
            fprintf(stdout," [-j] option only accepts [HIGH/LOW]\n");
            joinlevel = CN_NONE;
         }
         fprintf(stdout,"   Commands: joinlevel       = %d\n",joinlevel);
         continue;
         }
      if (strcmp(argv[i],"-label") == 0) {        /* label */
         contlabel = OFF;
         fprintf(stdout,"   Commands: contlabel       = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-lines") == 0) {        /* linetypes */  
         if (++i >= argc) usage(progname);
         linetypes = atoi(argv[i]);
         if (linetypes <=0 || linetypes >= 4) {
            linetypes = 2;
            fprintf(stdout,"   Warning : Invalid no of linetypes -");
            fprintf(stdout," linetypes reset to %d\n",linetypes);
         }
         fprintf(stdout,"   Commands: linetypes       = %d\n",linetypes);
         continue;
         }
      if (strcmp(argv[i],"-slice") == 0) {       /* output contour */  
         if (++i >= argc) usage(progname);
         cont_level = atof(argv[i]);
         print_slice= ON;
         fprintf(stdout,"   Commands: contour slice   = %f\n",cont_level);
         continue;
         }
      if (strcmp(argv[i],"-zlog") == 0) {         /* zlog       */
         zlog   = ON;
         fprintf(stdout,"   Commands: zlog            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-3D") == 0) {           /* 3D input format */
         if (++i >= argc) usage(progname);
         fprintf(stdout,"   Commands: Reading 3D data from \"%s\"\n",argv[i]);
         FILEFOUND = TRUE;
         read_contour_tria(argv[i]);
         continue;
         }
      if (strcmp(argv[i],"-2D") == 0) {           /* 2D input format */
         if (++i >= argc) usage(progname);
         fprintf(stdout,"   Commands: Reading 2D data from \"%s\"\n",argv[i]);
         FILEFOUND = TRUE;
         read_plot2D_file(argv[i]);
         continue;
         }
      if (strcmp(argv[i],"-o") == 0) {            /* options file */
         if (++i >= argc) usage(progname);
         if ((optionfile =  fopen(argv[i],"r")) == NULL) {
            fprintf(stderr,"cat: can't open %s\n", argv[i]);
            usage(progname);
         } else {
            fprintf(stdout,"   Commands: options-file    = \"%s\"\n",argv[i]);
            parse(optionfile);
            fclose(optionfile);
         }
         continue;
         }
      if (argv[i][0] == '-') usage(progname);
      if (argv[i][0] != '-' && argv[i][0] != '=') {
         fprintf(stdout,"   Commands: Reading mesh data from \"%s\"\n",argv[i]);
         FILEFOUND = TRUE;
         read_contour_mesh(argv[i]);
         zlog = FALSE;
         linetypes = 2;
         }
   }
   if (FILEFOUND == FALSE) usage(progname);
}


/* 
 * Read the contour data in rectangular mesh format 
 */
read_contour_mesh(filename)
char *filename;
{
   extern CNplotsetptr cont_plothead, cont_plottail;
   extern double cstep;
   extern int    nsteps, linetypes, zlog, plotID;
   extern int    joinlevel;
   CNplotsetptr pptr;
   int binary = FALSE;

   /* Read the contour data from a file */
   pptr = CNread_contourfile(filename, binary, joinlevel,
                             cstep, nsteps, linetypes, zlog, plotID++);

   /* Store the plotset in a global structure */
   if (pptr != NULL) CNstore_plotset(&cont_plothead, &cont_plottail, pptr, 0);
}


/* 
 * Read the contour data in triangular mesh format 
 */
read_contour_tria(filename)
char *filename;
{
   extern CNplotsetptr cont_plothead, cont_plottail;
   extern double cstep;
   extern int    nsteps, linetypes, zlog, plotID;
   CNplotsetptr pptr;
   int binary = FALSE;

   /* Read the contour data from a file */
   pptr = CNread_3Dcontourfile(filename, binary, 
                               cstep, nsteps, linetypes, zlog, plotID++);

   /* Store the plotset in a global structure */
   if (pptr != NULL) CNstore_plotset(&cont_plothead, &cont_plottail, pptr, 0);
}


/* 
 * Read the 2D plot data 
 */
read_plot2D_file(filename)
char *filename;
{
   extern CNplotsetptr cont_plothead, cont_plottail;
   extern int          xlog, ylog, plotID;
   CNplotsetptr pptr;
   int binary = FALSE;

   /* Read the contour data from a file */
   pptr = CNread_plot2Dfile(filename, binary, xlog, ylog, plotID++);

   /* Store the plotset in a global structure */
   if (pptr != NULL) CNstore_plotset(&cont_plothead, &cont_plottail, pptr, 0);
}


/* 
 * Print the acceptable command-line options and quit
 */
usage (program)
char *program;
{
   fprintf(stderr,"usage: %s [-o options_file] [-Pprinter] [-s scale] \
[-cstep step-size] [-nsteps #steps] [-lines #linetypes] [-zlog] \
[-exp] [-expand_mesh] [-expand_cont] [-join joinlevel] [-label] \
[-slice level] \
[-tl \"toplabel\"] [-xl \"xlabel\"] [-yl \"ylabel\"] \
[-e] [-g] [-l] [-clip] \
[-ps] [-print] [-noplot] \
[-xt #xticks] [-yt #yticks] \
[-xlog] [-ylog] \
[-xar] [-yar] [-ar] [-xyratio ratio] \
[-2D 2Dplot_file] [-3D 3Dcontour_file] contour_file ...\n",program);
   exit(-1);
}
