/*
 * array.c - subroutines to dynamically allocate space for an array 
 */

#include <stdio.h>
#include "plot2D.h"

/*************************************************************/
/***           Routines for 1D "double" arrays             ***/
/*************************************************************/

/* 
 * Allocate room for nsize elements of type double in a 1D array 
 */
double *CNcreate_1D_double_array(size)
int size;
{
   char   *malloc();
   double *newptr;
   unsigned int arr_size;
   
   /* figure out total number of elements */
   arr_size = (unsigned)(size * sizeof(double));
   
   /* allocate space using malloc(), and return a pointer */
   if ( (newptr = (double *)malloc(arr_size)) == NULL) {
     fprintf(stderr,"Out of memory\n");
     exit(-1);
   }

   /* return the pointer */
   return(newptr);
}


/* 
 * set the value of an array element
 */
void CNset_1D_double_array_value(arrptr,i,isize,val)
double *arrptr,*val;
int i,isize;
{
   if (i<0 || i>=isize) {
      fprintf(stderr,"Element [%d] is out of bounds!\n",i);
      exit(-1);
   }
   *(arrptr + i) = *val;
}


/*
 * get the value of an array element
 */
double CNget_1D_double_array_value(arrptr,i,isize)
double *arrptr;
int i,isize;
{
   double val;
   if (i<0 || i>=isize) {
      fprintf(stderr,"Element [%d] is out of bounds!\n",i);
      exit(-1);
   }
   val = *(arrptr + i);
   return(val);
}


/*
 * free the array
 */
void CNfree_1D_double_array(arrptr)
double *arrptr;
{
   /* free the array */
   free((char *)arrptr);
}


/*
 * find the min and max values of the array
 */
void CNget_1D_double_array_maxmin(arrptr,isize,min,max)
double *arrptr;
int    isize;
double *min, *max;
{
   double CNget_1D_double_array_value();
   double z;
   int    i;

   *min =  CN_LARGE;
   *max = -CN_LARGE;
   for (i=0; i<isize; i++) {
      z = CNget_1D_double_array_value(arrptr,i,isize);
      if (z < *min) *min = z;
      if (z > *max) *max = z;
   }
}


/*************************************************************/
/***           Routines for 2D "data" arrays               ***/
/*************************************************************/

/* 
 * assign array[i][j] 
 * i_size, j_size are the maximum dimensions of the 2D array.
 */
CNdatapt *CNcreate_data_array(isize,jsize)
int isize, jsize;
{
   char     *malloc();
   CNdatapt *newptr;
   unsigned int arr_size;

   /* figure out total no of elements */
   arr_size = (unsigned)(isize*jsize)*sizeof(CNdatapt);

   /* allocate space using malloc(), and return a pointer */
   if ( (newptr = (CNdatapt *)malloc(arr_size)) == NULL) {
     fprintf(stderr,"Out of memory\n");
     exit(-1);
   }

   /* return the pointer */
   return(newptr);
}


/*
 * Assign a value to an array element 
 */
void CNset_data_array_value(arrptr,i,j,isize,jsize,val)
CNdatapt *arrptr,*val;
int      i,j,isize,jsize;
{
   if (i<0 || i>=isize || j<0 || j>=jsize) {
      fprintf(stderr,"Element [%d][%d] is out of bounds!\n",i,j);
      exit(-1);
   }
   *(arrptr + i*jsize + j) = *val;
}


/*
 * Get a value from an array element 
 */
CNdatapt CNget_data_array_value(arrptr,i,j,isize,jsize)
CNdatapt *arrptr;
int i,j,isize,jsize;
{
   CNdatapt val;
   if (i<0 || i>=isize || j<0 || j>=jsize) {
      fprintf(stderr,"Element [%d][%d] is out of bounds!\n",i,j);
      exit(-1);
   }
   val = *(arrptr + i*jsize + j);
   return(val);
}


/*
 * Free the data array
 */
void CNfree_data_array(arrptr)
CNdatapt *arrptr;
{
   /* free the array */
   free((char *)arrptr);
}
