.TH CONTOUR L "11 July 1991"
.SH NAME
contour - contour plot program for X-windows and Postscript
.SH SYNOPSIS
.B
contour
[-o \fIoptions-file\fP]
[-P\fIprinter\fP]
[-s \fIscale\fP] 
[-cstep \fIstep-size\fP] [-nsteps \fI#steps\fP] [-lines \fI#linetypes\fP] 
[-zlog] 
[-exp] 
[-expand_mesh] 
[-expand_cont] 
[-join \fIjoinlevel\fP] 
[-label]
[-slice \fIlevel\fP] 
[-tl \fP"toplabel"\fP]
[-xl \fP"xlabel"\fP]
[-yl \fP"ylabel"\fP]
[-e]
[-g]
[-l]
[-clip]
[-ps]
[-print]
[-noplot]
[-xt \fI#xticks\fP]
[-yt \fI#yticks\fP]
[-xlog]
[-ylog]
[-xar]
[-yar]
[-ar]
[-xyratio \fIxyratio\fP]
[-3D 3Dcontour_file] contour_file contour-file ...
.SH DESCRIPTIONS
.PP
\fBContour\fP is a program for drawing contour plots on X11 windows.
The program will also produce a POSTSCRIPT plot which
can be dumped out to an APPLE Laserwriter or equivalent
POSTSCRIPT-compatible printer. 
\fBContour\fP reads in
data on a 3D surface from a \fIcontour-file\fP and
manipulates that data according to options specified
either in the command-line or in an \fIoptions-file\fP.
The \fIplot-file\fP can be compressed (see \fBcompress(1)\fP);
compressed files will be uncompressed automatically.
The program then draws contours on a screen or dumps the contours 
to a POSTSCRIPT file.  
.SH DATA FORMATS
.PP
The 3D surface input data
consists of z-values of the 3D surface, arranged on a
regular rectangular grid of size \fI(xmax - xmin)\fP \(mu \fI(ymax - ymin)\fP. 
The data file format is shown below.
.RS
.LP
.B
Data File Format (contour-file)
.LP
.nf
 xmin xmax ymin ymax 
 nxpts nypts
 z1
 z2
 ..
 ..
 ..
.fi
.RE
.LP
In the above, \fIxmin\fP, \fIxmax\fP, \fIymin\fP and \fIymax\fP are
lower and upper bounds of the grid,
and \fInxpts\fP and \fInypts\fP are the number of
grid divisions in x and y.  The z-values are arranged by column and row, i.e.
.RS
.LP
.nf
 z[0,0], z[0,1], z[0,2], ... z[0,j]
 z[1,0], z[1,1], z[1,2], ... z[1,j]
 ...
 z[i,0], z[i,1], z[i,2], ... z[i,j]
.fi
.RE
As an example, the data-set shown below defines a rectangular grid bounded
by \fIxmin\fP=0.0, \fIxmax\fP=2.0, \fIymin\fP=0.0 and \fIymax\fP=1.0.
There are 5 grid-divisions in \fIx\fR and 3 in \fIy\fP, corresponding to a
square grid of 0.5 units in width.  The first z-value specified corresponds
to (x,y)=(0.0,0.0), and z(0.5,1.0)=2.0 (third row, 2nd column).
.RS
.LP
.nf
 0.0 2.0 0.0 1.0
 5 3
 1.0 1.0 1.0 1.0 1.0
 1.0 2.0 1.0 2.0 1.0
 1.0 2.0 1.0 2.0 1.0
.fi
.RE
.LP
The rectangular mesh above is subdivided into smaller triangles for the
contour calculation.
.KS
.LP
\fBContour\fP also accepts as input a list of triangles, specified in
\fBpdraw(L)\fP format, i.e.,
.RS
.LP
.B
Data File Format (3D plot-file)
.LP
.nf
 xmin xmax ymin ymax zmin zmax
 ncurves
 4.0
 x1 y1 z1
 x2 y2 z2
 x3 y3 z3
 x1 y1 z1
 4.0
 x1 y1 z1
 x2 y2 z2
 x3 y3 z3
 x1 y1 z1
 ..
 ..
.fi
.RE
.LP
The files in this data-format are specified using the [-3D] flag in the
command-line.  Note that each file in this data-format \fBmust\fP be
preceded by the [-3D] flag. 
.KE
.SH MODUS OPERANDI
.LP
Upon starting up the program, \fBcontour\fP will read in the data stored in
the \fIcontour-file\fP and will then find the maximum and minimum z-values
of the surface.  It will then prompt for a contour step-size (i.e. the
contour increments), and read in any plotting options specified either in
the command line or in the \fIoptions-file\fP.
The contour step-size or number of contours may also be defined in the
command-line using the [-cstep] or [-nsteps] options.
The plot will then be drawn on the screen if possible.
The X11-plot is killed by clicking in the plot window with the mouse, or by
pressing the keyboard character "q" in the plot window.
Upon exiting X11, the user will be
prompted as to whether or not the POSTSCRIPT plot is to be sent to a printer.
Note that if the program is unable to open the X11 window (e.g DISPLAY not
set), it will automatically write a POSTSCRIPT plot to a file.
.LP
More than one input data-file may be specified on the commandline.
The [-cstep], [-nsteps], [-lines] and [-zlog] commandline options may be
respecified for each input file.  For example, 
.RS
.LP
.nf
contour -cstep 0.2 -zlog infile0 -cstep 0.1 infile1
.fi
.RE
overlays a log plot on a linear plot. 
.SH OPTIONS
.TP 5
.B \-o \fIoptions-file\fP
reads plotting options from the file \fIoptions-file\fP.
Each option specification consists of a keyword and its corresponding value.
The parser recognizes only a limited set of keywords; their values are
either numbers, quoted strings, or the words "on","off","true","false".
Keywords may be in upper or lower case.
All the words in the option specification must be on the same line.
The pound sign (#) indicates that the remainder of the line is a comment to
be ignored by the parser.
.RS
.LP
.B
List of Options (options-file)
.LP
.ta 1.5i 3.5i 4.5i
xlabel "LABEL"	#[default = "X-AXIS"]	- for the x-label   
.br
ylabel "LABEL"	#[default = "Y-AXIS"]	- for the y-label
.br
toplabel "LABEL"	#[default = "CONTOUR PLOT"]	- for the top-label  
.br
printer "PRINTER"	#[default = $PRINTER]	- define the printer
.br
grid on/off	#[default = off]	- draws a grid   
.br
equalscale on/off	#[default = on]	- for equal x-y scaling 
.br
landscape on/off	#[default = off] 	- print in landscape mode
.br
clip on/off	#[default = on]	- clips the picture
.br
postscript on/off	#[default = on]	- for postscript (PS) plot 
.br
printplot on/off	#[default = off]	- print PS file automatically 
.br
noplot on/off	#[default = off]	- don't draw graphics plot 
.br
xticks [1 - 20]	#[default = 4]	- no. of x-divisions  
.br
yticks [1 - 20]	#[default = 4]	- no. of y-divisions   
.br
xlog on/off	#[default = off]	- x-axis in log-scale
.br
ylog on/off	#[default = off]	- y-axis in log-scale
.br
autorange on/off	#[default = on]	- automatic axis ranging
.br
xautorange on/off	#[default = on]	- automatic x-axis ranging
.br
yautorange on/off	#[default = on]	- automatic y-axis ranging
.br
scale [0.1 - 1.0]	#[default = 1.00]	- scales the PS plot  
.br
cstep [step-size]	#[default = 0.0]	- contour step size
.br
nsteps [1 - 50]	#[default = 0]	- number of contour steps
.br
expand_cont on/off	#[default = off]	- draw 3D contours
.br
expand_mesh on/off	#[default = off]	- draw 3D mesh
.br
expand on/off	#[default = off]	- draw 3D contours and mesh
.br
contlabel on/off	#[default = on]	- for contour labels
.br
joinlevel high/low	#[default = off]	- for joining open curves
.br
linetypes [1 - 3]	#[default = 2]	- no. of contour linetypes
.br
slice [contour-level]	#[default = off] 	- single contour slice
.RE
.ta 1i 2i 3i 4i 5i 6i
.TP 5
.B \-P\fIprinter\fP
specifies which printer to which to send the postscript plot.
The current default sets the printer name to the environment variable
$PRINTER.
If this variable is not set, then the printer used is
the lp550M printer in 550M Cory (or whatever printer is hardcoded during
compile-time).
.TP 5
.B \-s \fIscale\fP
sets a plot scale factor.  This is used only for POSTSCRIPT plotting.
.TP 5
.B \-cstep \fIstep-size\fP
defines the contour step-size.  If this is not defined, the program prompts
for the step-size.
This option may be respecified for each input data-file.
.TP 5
.B \-nsteps \fInumber of steps\fP
defines the number of contour steps.  
If this is not defined, the program prompts for the step-size.
This option may be respecified for each input data-file.
.TP 5
.B \-lines \fIlinetypes\fP
defines the number of linetypes for the contour plot.  Valid values range
from 1-3.  The default is to use 2 linetypes; major contours are solid while
minor contours are dashed.
This option may be respecified for each input data-file.
.TP 5
.B \-zlog
informs the program that the z-values specified in the input file are to be
plotted in logarithmic scale.  Use this option to plot data with large
range, e.g. 1.0e10 < z < 1.0e15.
This option may be respecified for each input data-file.
.TP 5
.B \-exp
is equivalent to specifying both -expand_cont and \-expand_mesh.
.TP 5
.B \-expand_cont
prints out the 3D (x,y,z) values of the contours to the file
\fIimage.cont.3D\fP.  If the 3D plotting program \fBpdraw(L)\fP is available, 
\fBcontour\fP will make a system call to \fBpdraw\fP to plot this data-file.
.TP 5
.B \-expand_mesh
prints out the 3D (x,y,z) values of the triangular mesh (used internally for
contour calculation) to the file
\fIimage.cont.3D\fP.  If the 3D plotting program \fBpdraw(L)\fP is available, 
\fBcontour\fP will make a system call to \fBpdraw\fP to plot this data-file.
.TP 5
.B \-j \fIjoinlevel\fP
causes contour curves to be joined where possible.  This is done by defining
a boundary layer around the rectangular border, and setting the z-value of
that boundary layer at either the maximum or minimum z-value.
\fIjoinlevel\fP = \fIHIGH\fP or \fIhigh\fP sets the border z-value to its
maximum value: this is useful for plots which have high average z-values.
\fIjoinlevel\fP = \fILOW\fP or \fIlow\fP sets the border z-value to its
minimum value: this is useful for plots which have low average z-values.
.TP 5
.B \-label
suppresses the contour labels.
.TP 5
.B \-slice \fIlevel\fP
forces the program to compute the contours at a single 
value of z, specified by \fIlevel\fP.
The contours will be written to the file \fIimage.cont\fP.
The output data is organized in \fBdrawplot(L)\fP plot format.
.TP 5
.B \-xlabel \fI"xlabel"\fP
.TP 5
.B \-ylabel \fI"ylabel"\fP
.TP 5
.B \-tlabel \fI"toplabel"\fP
specifies the strings for the plot labels.
.TP 5
.B \-e
causes the x and y grids to be drawn with \fInon\fP-equal scales.
The default is to have equal-scales, i.e. the x and y lengths are equal for
equal x and y units.
.TP 5
.B \-g
forces a grid to be drawn.
.TP 5
.B \-l
prints the postscript plot in landscape mode (rotated).
.TP 5
.B \-clip
forces \fIno\fP clipping within the plot boundary.
The default is to automatically clip at the boundary.
.TP 5
.B \-print
sends the POSTSCRIPT lot to the printer automatically.
.TP 5
.B \-ps
turns off the postscript plotting mode.
This can also be done by setting the POSTSCRIPT environment variable to OFF.
e.g. % setenv POSTSCRIPT off
.TP 5
.B \-noplot
prevents plots on the graphics display.
This, with the -print and -cstep options, 
is very useful for batch-mode operations:
the program draws a postscript file and sends it to the printer
automatically without user-intervention.
.TP 5
.B \-xticks \fIxticks\fP
sets the number of ticks on the x-axis to \fIxticks\fP.
This option also turns off automatic ranging of the x-axis.
.TP 5
.B \-yticks \fIyticks\fP
sets the number of ticks on the y-axis to \fIyticks\fP.
This option also turns off automatic ranging of the y-axis.
.TP 5
.B \-ar
forces no automatic ranging of the x and y axes.
Without this option, automatic ranging is set ON, and the program
tries to select the best axis-scales possible for both the x and y-axes.
.TP 5
.B -xar
forces no automatic ranging of the x axis.
Without this option, automatic ranging is set ON for the x-axis,
and the program tries to select the best axis-scales possible for the
x-axis.
The x-autorange option is also turned off when \fIxticks\fP is defined
in the \fIoption-file\fP or in the commandline.
.TP 5
.B -yar
forces no automatic ranging of the y axis.
Without this option, automatic ranging is set ON for the y-axis,
and the program tries to select the best axis-scales possible for the
y-axis.
The y-autorange option is also turned off when \fIyticks\fP is defined
in the \fIoption-file\fP or in the commandline.
.TP 5
.B \-xlog
.TP 5
.B \-ylog
draws the x-axis or y-axis on a log scale.  NOTE: these options do \fBnot\fP
work correctly yet!
.TP 5
.B \-xyratio \fIxyratio\fP
is very useful for setting the plot size ratio, i.e. the ratio of the
plot-height (y) to the plot-width (x).  Normally, \fIxyratio\fP=0.75, 
but this is
ignored because the program tries to recompute the size of the plot to
satisfy equalscaling of the x and y axes.  However, when -xyratio is
specified in the command-line, equalscaling is turned off, and the plot is
sized as described above.  One application of this option is if the user
wants to fit a plot into a 6inch by 3inch area; the user then specifies
"contour -xyratio 0.5" and adjusts the scale factor until the plot fits.
.TP 5
.B \-3D 
specifies the plotfile to be of \fBpdraw(L)\fP format.  As such, the
plotfile should consist of triangles only.
For example, "contour -3D curves.plot.3D" will produce contour plots of the
3D plotfile \fIcurves.plot.3D\fP.
.TP 5
.I host:display
opens a window on the given host and display
.TP 5
.B \-d \fIhost:display\fP
.TP 5
.B \=\fIgeometry\fP 
.TP 5
.B \-geom \fIgeometry\fP
.TP 5
.B \-rv 
.TP 5
.B \-bw \fIborder-width\fP
.TP 5
.B \-bd \fIborder-color\fP 
.TP 5
.B \-fg \fIforeground-color\fP 
.TP 5
.B \-bg \fIbackground-color\fP
.TP 5
.B \-fn \fIfont-name\fP
sets input options for the X-window system.
.SH BUGS
Xlog and ylog options don't work yet; this is more due to the problems of
specifying and interpolating on a logarithmic mesh.  Also sparse data-sets
(contour meshes with few gridpoints) produce jagged curves, due to the
linear interpolation.  Contour meshes must be regular; there should be a way
to specify non-regular rectangular meshes.
.SH AUTHOR
Kenny K.H. Toh (ktoh@td2cad.intel.com)
.SH FILES
\fIdataplot.ps\fP 	temporary POSTSCRIPT file
.br
\fIimage.cont\fP 	file produced by -slice \fIlevel\fP option
.br
\fIimage.cont.3D\fP 	file produced by -exp, -expand_cont, -expand_mesh options.
.SH SEE ALSO
drawplot(L), pdraw(L)
