/*
 * expand.c - routines to write out the 3D triangles into a 3D format
 * suitable for 3D plotting 
 */

#include <stdio.h>
#include "plot2D.h"


/* Write the contents of Pptr plotset (mesh) to the outputfile */
void CNexpand_mesh(Pptr,outputfile)
CNplotsetptr Pptr;
char         *outputfile;
{
   FILE      *fopen(), *fo;
   int       ncurves=0;
   CNtriaptr T;

   /* Open the outputfile */
   if ((fo =  fopen(outputfile,"w")) == NULL) {
      fprintf(stderr,"cat: can't open %s\n", outputfile);
      exit(1);
   }

   /* count the number of triangles */
   for (T=Pptr->triahead; T!=NULL; T=T->next) ncurves++;

   /* print out header - xmin,xmax,ymin,ymax,zmin,zmax,ncurves */
   fprintf(fo,"%10.5f %10.5f  ",Pptr->xmin,Pptr->xmax);
   fprintf(fo,"%10.5f %10.5f  ",Pptr->ymin,Pptr->ymax);
   fprintf(fo,"%10.5f %10.5f\n",Pptr->zmin,Pptr->zmax);
   fprintf(fo,"%10.5f\n",(double)ncurves);

   /* now print out the triangles */
   for (T=Pptr->triahead; T!=NULL; T=T->next) {
      fprintf(fo,"%10.5f\n",4.0);
      fprintf(fo,"%10.5f %10.5f %10.5f\n",T->pt1.x,T->pt1.y,T->pt1.z);
      fprintf(fo,"%10.5f %10.5f %10.5f\n",T->pt2.x,T->pt2.y,T->pt2.z);
      fprintf(fo,"%10.5f %10.5f %10.5f\n",T->pt3.x,T->pt3.y,T->pt3.z);
      fprintf(fo,"%10.5f %10.5f %10.5f\n",T->pt1.x,T->pt1.y,T->pt1.z);
   }      

   /* now close the file */
   fclose(fo);
   fprintf(stdout,"   %d Triangles saved in \"%s\"\n",ncurves,outputfile);
}

/* Write the contents of Pptr plotset (contours) to the outputfile */
void CNexpand_cont(Pptr,outputfile)
CNplotsetptr Pptr;
char         *outputfile;
{
   FILE       *fopen(), *fo;
   int        ncurves=0,npts;
   CNcurveptr C;
   CNnodeptr  N;

   /* Open the outputfile */
   if ((fo =  fopen(outputfile,"w")) == NULL) {
      fprintf(stderr,"cat: can't open %s\n", outputfile);
      exit(1);
   }

   /* count the number of triangles */
   for (C=Pptr->curvehead; C!=NULL; C=C->next) ncurves++;

   /* print out header - xmin,xmax,ymin,ymax,zmin,zmax,ncurves */
   fprintf(fo,"%10.5f %10.5f  ",Pptr->xmin,Pptr->xmax);
   fprintf(fo,"%10.5f %10.5f  ",Pptr->ymin,Pptr->ymax);
   fprintf(fo,"%10.5f %10.5f\n",Pptr->zmin,Pptr->zmax);
   fprintf(fo,"%10.5f\n",(double)ncurves);

   /* now print out the triangles */
   for (C=Pptr->curvehead; C!=NULL; C=C->next) {

      /* Count the number of points */
      npts = 0;
      for (N=C->nodehead; N!=NULL; N=N->next) npts++;
      fprintf(fo,"%10.5f\n",(double)npts);

      /* Print out the points */
      for (N=C->nodehead; N!=NULL; N=N->next)  
         fprintf(fo,"%10.5f %10.5f %10.5f\n",N->pt.x,N->pt.y,N->pt.z);
   }      

   /* now close the file */
   fclose(fo);
   fprintf(stdout,"   %d Contour Curves saved in \"%s\"\n",ncurves,outputfile);
}


/* Print out a single contour */
void CNcalc_single_cont(Pptr,cont_level,outputfile)
CNplotsetptr Pptr;
double       cont_level;
char         *outputfile;
{
   FILE         *fopen(), *fo;
   int          npts=0,ncurves=0;
   CNnodeptr    N;
   CNcurveptr   C, curvehead, curvetail;

   /* print out info about the image read in */
   fprintf(stdout,"\n   Slicing the dataset at z = %f\n",cont_level);
   fprintf(stdout,"   Dataset Information:\n");
   fprintf(stdout,"      PLOT ID   = %d\n",Pptr->ID);
   fprintf(stdout,"      Origin    = %s\n",Pptr->filename);
   fprintf(stdout,"      Linetypes = %d\n",Pptr->linetypes);
   fprintf(stdout,"      Z-Log     = %s\n",((Pptr->zlog) ? "TRUE" : "FALSE"));
   fprintf(stdout,"      Maximum z-value        = %f\n",Pptr->zmax);
   fprintf(stdout,"      Minimum z-value        = %f\n",Pptr->zmin);

   /* check the desired contour level */
   if ((Pptr->zmin > cont_level) || (cont_level > Pptr->zmax)) {
      fprintf(stderr,"   ***Error : The desired contour level %.2f",cont_level);
      fprintf(stderr," is not within the surface\n");
   } else {
      /* Save the curve list first */
      curvehead = Pptr->curvehead;
      curvetail = Pptr->curvetail;
      Pptr->curvehead = NULL; 
      Pptr->curvetail = NULL;

      /* now find the desired contours and print it out */
      CNfind_contour(Pptr,cont_level,0);

      /* open up the file */
      if ((fo = fopen(outputfile,"w")) == NULL) {
         fprintf(stderr,"cat: can't open %s\n",outputfile);
         exit(1);
      }

      /* count the number of triangles */
      for (C=Pptr->curvehead; C!=NULL; C=C->next) ncurves++;

      /* print out header - xmin,xmax,ymin,ymax,zmin,zmax,ncurves */
      fprintf(fo,"%10.5f %10.5f  ",Pptr->xmin,Pptr->xmax);
      fprintf(fo,"%10.5f %10.5f\n",Pptr->ymin,Pptr->ymax);
      fprintf(fo,"%10.5f\n",(double)ncurves);

      /* now print out the curves */
      for (C=Pptr->curvehead; C!=NULL; C=C->next) {

         /* Count the number of points */
         npts = 0;
         for (N=C->nodehead; N!=NULL; N=N->next) npts++;
         fprintf(fo,"%10.5f\n",(double)npts);

         /* Print out the points */
         for (N=C->nodehead; N!=NULL; N=N->next)
            fprintf(fo,"%10.5f %10.5f\n",N->pt.x,N->pt.y);
      }

      /* Close the file */
      fclose(fo);

      /*
       * delete the plotset 
       */
      /* delete all the curveptrs in P */
      while (Pptr->curvehead !=NULL)
      CNdelete_curveptr(&(Pptr->curvehead),&(Pptr->curvetail),Pptr->curvehead);

      /* Put back the original curves */
      Pptr->curvehead = curvehead;
      Pptr->curvetail = curvetail;

      /* finished */
      fprintf(stdout,"   The x-y points of the %.2f contour ",cont_level);
      fprintf(stdout,"has been printed in \"%s\"\n",outputfile);
   }
}

