/*
 *  init.c - initialize data
 */

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"

/* Initialize some data */
initialize_data(pgname)
char *pgname;
{
   char   *getenv();
   char   *ptr;

#ifndef PRINTER_NAME
   static char *printer_name = "lp550M";
# else
   static char *printer_name = PRINTER_NAME;
#endif

   /* Program name */
   progname     = pgname;

   /* Terminal Type */
   term = getenv("TERM");
   fprintf(stdout,"   Terminal type             = %s\n",term);

   /* Labeling information */
   strcpy(xlabel   , "X-AXIS");
   strcpy(ylabel   , "Y-AXIS");
   strcpy(toplabel , "CONTOUR PLOT");

   /* Linked list data-structure */
   cont_plothead = NULL;
   cont_plottail = NULL;
   plotID        = 0;

   /* Contour-specific options */
   expand_cont  = OFF;
   expand_mesh  = OFF;
   print_slice  = OFF;
   cont_level   = 0.0;
   joinlevel    = CN_NONE;
   contlabel    = ON;
   linetypes    = 2;
   cstep        = 0.0;
   nsteps       = 0;
   xlog         = OFF;
   ylog         = OFF;
   zlog         = OFF;

   /* Plotting parameters/options */
   grid         = OFF;
   equalscale   = ON;
   landscape    = OFF;
   clip         = ON;
   postscript   = ON;
   printplot    = OFF;
   noplot       = OFF;
   xautorange   = ON;
   yautorange   = ON;
   autorange    = ON;
   scale        = 1.00;
   xyratio      = 0.75;
   xticks       = 4;
   yticks       = 4;

   /* initialize general X arguments */
   font_name    = NULL;
   geometry     = NULL;
   display_name = NULL;
   border_color = NULL;
   fore_color   = NULL;
   back_color   = NULL;
   mouse_color  = NULL;
   icon_color   = NULL;
   reverse      = 0;
   border_width = 0;

   /* check the POSTSCRIPT environment variable */
   if ((ptr = getenv("POSTSCRIPT")) != NULL)
      if ((strcmp(ptr,"OFF")==0) || (strcmp(ptr,"off")==0)) postscript = OFF;

   /* printer type */
   if ((ptr = getenv("PRINTER")) == NULL)
#ifdef SYSV
      sprintf(printer,"-d%s",printer_name);
#else
      sprintf(printer,"-P%s",printer_name);
#endif
   else
#ifdef SYSV
      sprintf(printer,"-d%s",ptr);
#else
      sprintf(printer,"-P%s",ptr);
#endif
   fprintf(stdout,"   Default Printer           = %s\n",printer);
}

