/***  main.c  ***/

/* CONTOUR Version 3.0 : Kenny K.H. Toh, July 2, 1991 */

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"
#include "patchlevel.h"

main(argc,argv)
int argc;
char *argv[];
{
   extern int expand_mesh, expand_cont, print_slice;

   /* Print welcome message */
   fprintf(stdout,"Contour  V%d.%d  %s\n",VERSION,PATCHLEVEL,DATE);

   /* Initialize the variables */
   initialize_data(argv[0]);

   /* read the data and options */
   read_commandline(argc,argv);

   /* Go through all the data-sets and slice them up */
   slice_data();

   /* expand the plot if necessary */
   if (expand_mesh) cn_expand_mesh();
   if (expand_cont) cn_expand_cont();
   if (print_slice) cn_print_slice();

   /* Draw the picture */
   draw_view(argc,argv);

   /* End the program - this signifies normal exit */
   fprintf(stdout,"Congratulations!!!  You have been TERMINATED...\n");
   fflush(stdout);
}


/* 
 * Go through each data-set and slice it up
 */
slice_data()
{
   extern CNplotsetptr cont_plothead, cont_plottail;
   CNplotsetptr Pptr;

   /* Double-check the data to ensure there is something to plot */
   if (cont_plothead == NULL) {
      fprintf(stderr,"   ***Error! No data found in linked list\n");
      fprintf(stderr,"      Stopping program now.\n");
      exit(-1);
   }

   for (Pptr = cont_plothead; Pptr!=NULL; Pptr=Pptr->next) {
      /*
       * Slice the contours
       */
      if (Pptr->plottype == CN_CONTOUR)
         CNslice_contours(Pptr);
   }
}


/*
 * Draw a pretty picture
 */
draw_view(argc,argv)
int argc;
char **argv;
{
   FILE   *fopen(), *ips;
   extern CNplotsetptr cont_plothead, cont_plottail;
   extern double       xmin, xmax, ymin, ymax, zmin, zmax;
   extern int          postscript, printplot, noplot;
   char   command[MAXCHAR], c;


   /* Reset the contour boundaries */
   CNset_contour_boundaries(cont_plothead, cont_plottail,
                            &xmin, &xmax, &ymin, &ymax, &zmin, &zmax);

   /* now draw the plots */
   if (!noplot) {
      /*
       * Try to open the window first; proceed with plot if successful
       * Default is xterm
       */
      if (InitXWindow(argc,argv)) {
         /*
          * X procedures
          */
         printf("\n   Proceeding with X Plot...\n");
         RunOps();
      } 
   }

   /* Postscript plot */
   if (postscript == ON) {
      if ((ips = fopen(PSFILE,"w")) == NULL) {
         fprintf(stderr,"cat: can't open %s\n",PSFILE);
         exit(1);
      }
      printf("\n   Proceeding with POSTSCRIPT Plot...\n");
      initps(ips);
      axesps(ips);
      plotps(ips);
      endgrps(ips);
   }

   /* Get postscript printout */
   if (postscript == ON) {
      if (printplot == OFF) {
      /* prompt for a print out */
         fprintf(stdout,"   Type return (or n/q) to ");
         fprintf(stdout,"send the plot to the %s printer :",printer);
         if ( (c=getc(stdin)) == '\n' || c == 'y' || c == 'Y' ) {
            sprintf(command,"%s %s %s",PRINT,printer,PSFILE);
            system(command);
            fprintf(stdout,"   Postscript file has been ");
            fprintf(stdout,"sent to the %s printer\n",printer);
         } else if (c == 'q')
            exit(1);
         if (c != '\n') while ( (c=getc(stdin)) != '\n');
      } else {
         /* print directly */
         sprintf(command,"%s %s %s",PRINT,printer,PSFILE);
         system(command);
         fprintf(stdout,"   Postscript file has been ");
         fprintf(stdout,"sent to the %s printer\n",printer);
      }
   }
}


/* 
 * Print out a 2D slice and write it to a file in 2D plot (drawplot) format.
 */
cn_print_slice()
{
   extern  CNplotsetptr cont_plothead;
   extern  double       cont_level;

   /* slice the first data set */
   CNcalc_single_cont(cont_plothead,cont_level,CTFILE);

   return;
}


/* 
 * Expand the mesh provided and write it in 3D format.
 * Imagefile is the first (rectangular mesh) file specified
 * in the input command line 
 */
cn_expand_mesh()
{
   extern  CNplotsetptr cont_plothead;
   int     nosort=FALSE;

   /* expand the first data set */
   CNexpand_mesh(cont_plothead,TDFILE);

   /* if interactive, try to draw the 3D mesh using pdraw */
   if (!noplot) pdraw(TDFILE,nosort);

   return;
}

/* 
 * Write and try to draw the 3D contours.
 * Imagefile is the first (rectangular mesh) file specified
 * in the input command line 
 */
cn_expand_cont()
{
   extern  CNplotsetptr cont_plothead;
   int     nosort=FALSE;

   /* expand the first data set */
   CNexpand_cont(cont_plothead,TDFILE);

   /* if interactive, try to draw the 3D mesh using pdraw */
   if (!noplot) pdraw(TDFILE,nosort);

   return;
}

pdraw(plotfile,nosort)
char *plotfile;
int   nosort;
{
   char command[MAXCHAR];
   char args[MAXCHAR];

   if (PDRAW_IS_AVAILABLE) {
      strcpy(args," ");
      if (nosort) strcat(args,"-nosort ");
      sprintf(command,"%s %s %s",PDRAW,args,plotfile);
      system(command);
   }
}
