/*
 * parser.c - parses the input data based on keywords
 */

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"

#define BOOLEAN_VALUE(x) ((x) ? "TRUE" : "FALSE")
#define ERROR  -1


/* Define the keywords */

#define MAXKEY 29
enum keyvals {
   XLABEL,
   YLABEL,
   TLABEL,
   PRINTER,
   GRID,
   EQSCALE,
   LDSCAPE,
   CLIP,
   PSCRIPT,
   PRTPLOT,
   NOPLOT,
   XTICKS,
   YTICKS,
   XLOG,
   YLOG,
   ZLOG,
   XAUTOR,
   YAUTOR,
   AUTOR,
   SCALE,
   CNSTEP,
   NSTEPS,
   EXPAND,
   EXPANDC,
   EXPANDM,
   JOINLVL,
   CONTLBL,
   LINETYP,
   CNSLICE,
};
static char *keywords[] = {
   "xlabel",
   "ylabel",
   "toplabel",
   "printer",
   "grid",
   "equalscale",
   "landscape",
   "clip",
   "postscript",
   "printplot",
   "noplot",
   "xticks",
   "yticks",
   "xlog",
   "ylog",
   "zlog",
   "xautorange",
   "yautorange",
   "autorange",
   "scale",
   "cstep",
   "nsteps",
   "expand",
   "expand_cont",
   "expand_mesh",
   "joinlevel",
   "contlabel",
   "linetypes",
   "slice",
};


/*
 * Parser - go thru the lines searching for keywords and values
 */
parse(op) 
FILE *op;
{
   static int  getline();
   static int  getwords();
   static void assignvalue();

   int  len,match,i;
   char line[MAXCHAR];
   char name[MAXCHAR],value[MAXCHAR];
   char word[100][MAXCHAR];

   /* Go thru each line and check for keywords */
   while (len=getline(op,line, MAXCHAR) > 0)
      if (getwords(line,len,word) >= 2) {
         strcpy(name, word[0]);
         strcpy(value,word[1]);
         match = FALSE;
         i     = 0;
         while (i < MAXKEY && match==FALSE ) {
            CNstring_to_lower(name);
            if (strcmp(keywords[i],name) == 0) {
               match = TRUE;
               assignvalue(i,value);
            }
            i++;
         }
         if (match==FALSE) fprintf(stderr,"   ***%s: Unknown keyword\n",name);
      }
}


/*
 * Assign a value to a keyword 
 */
static void assignvalue(index,value) 
int index;
char *value;
{
   static void   assign_boolean_keyword();
   extern char   xlabel[MAXCHAR];
   extern char   ylabel[MAXCHAR];
   extern char   toplabel[MAXCHAR];
   extern char   printer[MAXCHAR];
   extern int    grid, equalscale, landscape, clip;   
   extern int    postscript, printplot, noplot;
   extern int    xticks, yticks, xlog, ylog, zlog;
   extern int    xautorange, yautorange, autorange;
   extern int    linetypes, contlabel, joinlevel;
   extern double scale;
   extern double cstep;

   switch (index) {
   case XLABEL :
        strcpy(xlabel , value); 
        fprintf(stdout,"   Options : Xlabel          = \"%s\"\n",xlabel);
        break;
   case YLABEL :
        strcpy(ylabel , value);
        fprintf(stdout,"   Options : Ylabel          = \"%s\"\n",ylabel);
        break;
   case TLABEL :
        strcpy(toplabel , value); 
        fprintf(stdout,"   Options : Top label       = \"%s\"\n",toplabel);
        break;
   case PRINTER:
        sprintf(printer,"-P%s",value);
        fprintf(stdout,"   Options : Printer         = \"%s\"\n",printer);
        break;
   case GRID   :
        assign_boolean_keyword(&grid,       value, "grid        ");
        break;
   case EQSCALE:
        assign_boolean_keyword(&equalscale, value, "equalscale  ");
        break;
   case LDSCAPE:
        assign_boolean_keyword(&landscape , value, "landscape   ");
        break;
   case CLIP   :
        assign_boolean_keyword(&clip      , value, "clip        ");
        break;
   case PSCRIPT:
        assign_boolean_keyword(&postscript, value, "postscript  ");
        break;
   case PRTPLOT:
        assign_boolean_keyword(&printplot , value, "printplot   ");
        break;
   case NOPLOT :
        assign_boolean_keyword(&noplot    , value, "noplot      ");
        break;
   case XTICKS  :
        xticks = atoi(value); 
        if (xticks <= 0 || xticks > 20) xticks = 4;
        fprintf(stdout,"   Options : xticks          = %d\n",xticks);
        xautorange =  OFF;
        break;
   case YTICKS  :
        yticks = atoi(value); 
        if (yticks <= 0 || yticks > 20) yticks = 4;
        fprintf(stdout,"   Options : yticks          = %d\n",yticks);
        yautorange =  OFF;
        break;
   case XLOG   :
        assign_boolean_keyword(&xlog      , value, "xlog        ");
        break;
   case YLOG   :
        assign_boolean_keyword(&ylog      , value, "ylog        ");
        break;
   case ZLOG   :
        assign_boolean_keyword(&zlog      , value, "zlog        ");
        break;
   case XAUTOR :
        assign_boolean_keyword(&xautorange, value, "xautorange  ");
        break;
   case YAUTOR :
        assign_boolean_keyword(&yautorange, value, "yautorange  ");
        break;
   case AUTOR  :
        assign_boolean_keyword(&autorange , value, "autorange   ");
        break;
   case SCALE   :
        scale = atof(value); 
        if (scale < 0.1 || scale > 1) scale = 1.0;
        fprintf(stdout,"   Options : scale           = %f\n",scale);
        break;

   case CNSTEP  :
        cstep = atof(value); 
        if (cstep > 1.0e-5) nsteps = 0;
        fprintf(stdout,"   Options : cstep           = %f\n",cstep);
        break;
   case NSTEPS  :
        nsteps = atoi(value); 
        if (nsteps > 0) cstep = 0.0;
        fprintf(stdout,"   Options : nsteps          = %d\n",nsteps);
        break;
   case EXPAND :
        assign_boolean_keyword(&expand_mesh, value, "expand_mesh ");
        assign_boolean_keyword(&expand_cont, value, "expand_cont ");
        break;
   case EXPANDC:
        assign_boolean_keyword(&expand_cont, value, "expand_cont ");
        break;
   case EXPANDM:
        assign_boolean_keyword(&expand_mesh, value, "expand_mesh ");
        break;
   case JOINLVL:
        CNstring_to_upper(value);
        if      (strcmp(value,"HIGH")==0) joinlevel = CN_HIGH;
        else if (strcmp(value,"LOW" )==0) joinlevel = CN_LOW; 
        else { 
           joinlevel = CN_NONE;
           fprintf(stdout,"   Options : Unknown keyword = \"%s\"\n",value);
           fprintf(stdout,"   Warning : Invalid keyword - ");
           fprintf(stdout,"joincurve [-j] [HIGH/high/LOW/low]\n");
           fprintf(stdout,"   Options : joinlevel       = %s\n","OFF");
        }
        if (joinlevel == CN_HIGH || joinlevel == CN_LOW)
           fprintf(stdout,"   Options : joinlevel       = %s\n",value);
        break;
   case CONTLBL:
        assign_boolean_keyword(&contlabel , value, "contlabel   ");
        break;
   case LINETYP : 
        linetypes = atoi(value);
        if (linetypes <=0 || linetypes >= 4) linetypes = 2;
        fprintf(stdout,"   Options : linetypes       = %d\n",linetypes);
        break;
   case CNSLICE :
        cont_level = atof(value);
        print_slice= ON;
        fprintf(stdout,"   Options : contour slice   = %f\n",cont_level);
        break;
   default: break;
   }
}


/* 
 * Assign a boolean value to a keyword 
 */
static void assign_boolean_keyword(keyval,value,label)
int  *keyval;
char *value,*label;
{
   static int string_to_boolean();
   int    intval;

   if ((intval = string_to_boolean(value))==ERROR)
      fprintf(stdout,"   Options : *** %s- Unknown keyvalue = \"%s\"\n",
      label,value);
   else
      *keyval = intval;
   fprintf(stdout,"   Options : %s    = %s\n", label, BOOLEAN_VALUE(*keyval));
}


/* 
 * Convert a character string into boolean
 */
static int string_to_boolean(word)
char *word;
{
   int val;

   /* convert the word to uppercase first, for easier comparisons */
   CNstring_to_upper(word);

   /* "T", "TRUE", "F", "FALSE", "ON", "OFF" are all accepted */
   if ((strncmp(word,"T",1)==0) || (strcmp (word,"ON")==0)) 
      val = TRUE;
   else if ((strncmp(word,"F",1)==0) || (strcmp (word,"OFF")==0))
      val = FALSE;
   else
      val = ERROR;
   return(val);
}


/* 
 * get line into s 
 */
static int getline(op,s,lim) 
FILE *op;
char s[];
int lim;
{
    int c,i,j;

    /* read the line until termination marks ';' or '#' */
    for (i=0; i<lim-1 && (c=getc(op))!=EOF && c!='\n'&&c!='#'&& c!=';' ; ++i) 
       s[i] = c;
    if (i==lim-1 || c==';' ||c=='#' || c=='\n') s[i++] = '\n'; 
    if (c==';'||c=='#')
       for (j=0; (c=getc(op))!=EOF && c!='\n'; ++j);
    s[i] = '\0';

    return(i);
}


 
/* 
 * Get the words in a line 
 */
static int getwords(s,length,word)  
char s[];
int length;
char word[100][MAXCHAR];
{
   int i,nc,nw,inword,collectword;
   char c;

   nc = nw = 0;
   inword = FALSE;
   collectword = FALSE;

   if (length==1 && s[0] == '\n') return(nw);
   for (i=0; s[i] != '\0'; i++) {
      c = s[i];
      if (c == ' ' || c=='\n' || c == '\t' || c=='=') {
         if (inword == TRUE && (collectword==FALSE || c=='\n') ) {
            word[nw++][nc++] = '\0';
            inword = FALSE;
            nc = 0;
         } else if (collectword == TRUE)
            word[nw][nc++] = c;
      } else if (c == '\'' || c == '"') {
         if (collectword == FALSE) collectword = TRUE; 
         else {
            collectword = FALSE; 
            word[nw++][nc++] = '\0';
            inword = FALSE;
            nc = 0;
         }
      } else {
         word[nw][nc++] = c;
         if (inword == FALSE) inword = TRUE;
      }
   }
   return(nw);
}

