/*
 * plotX11.c - all the X11 routines
 */

#include <stdio.h>
#include <strings.h>
#include <math.h>

#ifdef NO_X11PLOT
   /* No X11 plot in this case */

/*** X routines for machines without X ***/

int InitXWindow(argc,argv)
int argc;
char **argv;
{
   fprintf(stderr,"\nSorry, the X11 routines have not been compiled...\n");
   return(0);
}

RunOps()
{
}

#else
   /* Hey ho, lets do an X11 plot */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/keysym.h>

#include "header.h"
#include "iconX11.ic"

/* colors */
#define DEFAULT_BORDER_COLOR  "Blue"
#define DEFAULT_BACK_COLOR    "MidNightBlue"
#define DEFAULT_FORE_COLOR    "White"
#define DEFAULT_MOUSE_COLOR   "White"
#define DEFAULT_ICON_COLOR    "Red"
#define BDR_C                 0
#define BACK_C                1
#define FORE_C                2
#define MOUSE_C               3
#define ICON_C                4
#define LINE1_C               5
#define LINE2_C               6
#define LINE3_C               7
#define LINE4_C               8
#define LINE5_C               9
#define MAX_COLORS            10

unsigned long     border_pixel;                   /* color of border */
unsigned long     background_pixel;               /* color of background */
unsigned long     foreground_pixel;               /* color of foreground */
unsigned long     mouseground_pixel;              /* mouse cursor color */
unsigned long     icon_pixel;                     /* icon color */
unsigned long     colors[MAX_COLORS];             /* color pixel values */
char              *clr_name[MAX_COLORS];          /* color names */

/* line-types */
#  define DOTTED       2
#  define SHORT_DOTTED 2
#  define LONG_DOTTED  2
#  define DASHED       2
#  define DOT_DASHED   4
#  define DASH_DOTTED  4
#  define SHORT_DASHED 2
#  define LONG_DASHED  2
#  define EQUAL_DASHED 2
#  define ODD_DASHED   3
   static unsigned char dash_list_dotted[DOTTED]             = {1,6};
   static unsigned char dash_list_short_dotted[SHORT_DOTTED] = {1,3};
   static unsigned char dash_list_long_dotted[LONG_DOTTED]   = {1,8};
   static unsigned char dash_list_dashed[DASHED]             = {4,4};
   static unsigned char dash_list_dot_dashed[DOT_DASHED]     = {3,4,3,1};
   static unsigned char dash_list_dash_dotted[DASH_DOTTED]   = {4,3,1,3};
   static unsigned char dash_list_short_dashed[SHORT_DASHED] = {3,5};
   static unsigned char dash_list_equal_dashed[EQUAL_DASHED] = {6,6};
   static unsigned char dash_list_long_dashed[LONG_DASHED]   = {7,4};
   static unsigned char dash_list_odd_dashed[ODD_DASHED]     = {1,2,3};

/* plot information */
#define X_ORG            5
#define Y_ORG            5
#define X_DIM            600
#define Y_DIM            800
#define DEFAULT_BDR_DIM  75

int Xxdim, Xydim;
int Xxmin, Xymin, Xxmax, Xymax;
int width, height;

/* Misc */
#define DEFAULT_BORDER_WIDTH  3
#define Helv12B "-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1"
#define DEFAULT_FONT          Helv12B
#define ALT_FONT              "8x13"
#define SMALL_FONT            "6x10"

short gray_bits[16] = {
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555};

Display     *display;                   /* display */
int         screen;                     /* screen */
Window      window;                     /* parent window */
XFontStruct *font_info;                 /* font information */
XFontStruct *lblfont_info;              /* label font information */
XSizeHints  size_hints;                 /* window info for manager */
XWMHints    wm_hints;                   /* window info for manager */
GC          gc, gcl;                    /* graphics context */

int InitXWindow(argc,argv)
int  argc;
char **argv;
{
   XFontStruct *Set_Fonts();
   Pixmap      icon_pixmap;
   XGCValues   gcvalues;
   char        default_geometry[100];
   int         x, y;
   int         Xxwid, Xywid;
   double      ratio;
   extern      double xyratio;
   extern      int    equalscale;

   /* Open display first, and get screen info */
   if (!(display = XOpenDisplay(display_name))) {
      fprintf(stderr,"\n%s:Could not open X11 display %s\n",
              progname,display_name);
      return(0);
   }
   screen = DefaultScreen(display);

   /* Merge the options first */
   Merge_Options();

   /* allocate the colors */
   if (!Alloc_Colors())
      return(0);

   /* Set up the window size
    *
    * Xxdim, Xydim : Width and height of the window
    * Xxwid, Xywid : Width and height of the plot
    */
   Xxdim = X_DIM;
   Xydim = Y_DIM;
   Xxwid = Xxdim - 2*DEFAULT_BDR_DIM;
   Xywid = Xydim - 2*DEFAULT_BDR_DIM;
   if (equalscale == ON) {
      /* The plot must fit inside the window */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio < ((double)Xywid)/((double)Xxwid))
         Xywid  = Xxwid*ratio;
      else
         Xxwid  = Xywid/ratio;
   } else {
      /* The plot must fit inside the window */
      if (Xxwid*xyratio < Xywid)
         Xywid = Xxwid*xyratio;
      else
         Xxwid = Xywid/xyratio;
   }
   Xxdim = Xxwid + 2*DEFAULT_BDR_DIM;
   Xydim = Xywid + 2*DEFAULT_BDR_DIM;
   sprintf(default_geometry,"=%dx%d+%d+%d",Xxdim,Xydim,X_ORG,Y_ORG);

   /* Initialize the window */
   if (!XGeometry(display,screen,geometry,default_geometry,border_width,
                 0,0,0,0,&x,&y,&width,&height)) {
      x      = X_ORG;
      y      = Y_ORG;
      width  = Xxdim;
      height = Xydim;
   }
   window = XCreateSimpleWindow(display, RootWindow(display,screen),
                                x, y, width, height, border_width,
                                border_pixel, background_pixel);

   /* Initialize size hint property for window manager */
   size_hints.flags      = PPosition | PSize | PMinSize;
   size_hints.x          = x;
   size_hints.y          = y;
   size_hints.width      = width;
   size_hints.height     = height;
   size_hints.min_width  = 200;
   size_hints.min_height = 200;

   /* Create pixmap of depth 1 (bitmap) for icon */
   icon_pixmap = XCreateBitmapFromData(display, window, icon_bitmap_bits,
                        icon_bitmap_width, icon_bitmap_height);

   /* set properties for window manager */
   XSetStandardProperties(display,window, "Contour Program", "Contour",
                          icon_pixmap, argv, argc, &size_hints);

   /* Initialize input hint property for window manager */
   wm_hints.flags = InputHint;
   wm_hints.input = True;
   XSetWMHints(display, window, &wm_hints);

   /* Select event types wanted */
   XSelectInput(display,window,ExposureMask | ButtonPressMask  |
                KeyPressMask | StructureNotifyMask );

   /* Create GC for text and drawing */
   gc = XCreateGC(display, window, 0, &gcvalues);
   gcl= XCreateGC(display, window, 0, &gcvalues);

   /* Find a font and load it */
   font_info = Set_Fonts(display, gc, DEFAULT_FONT, ALT_FONT);
   if (font_info == NULL) {
      fprintf(stderr,"\nSorry - couldn't load any X11 fonts\n");
      fprintf(stderr,"Stopping X11 Plot.\n");
      return(0);
   }

   /* Find a (small) font and load it */
   lblfont_info = Set_Fonts(display, gcl, SMALL_FONT, ALT_FONT);
   if (lblfont_info == NULL) 
      lblfont_info = font_info;
   XSetFont(display, gcl, lblfont_info->fid);

   /* specify foreground */
   XSetForeground(display, gc, foreground_pixel);
   XSetForeground(display,gcl, foreground_pixel);

   /* Display the window */
   XMapWindow(display, window);

   /* return */
   return(1);
}

/*
 * Load a font
 */
static
XFontStruct *Load_Font(display, fontname)
Display *display;
char    *fontname;
{
   XFontStruct *font_info = NULL;

   /* find a font and load it */
   font_info = XLoadQueryFont(display,fontname);
   if (font_info == NULL)
      fprintf(stderr,"\nWarning : Could not open X11 font \"%s\"\n",fontname);

   /* return the font info */
   return(font_info);
}


/*
 * Set the font in a gc
 */
XFontStruct *Set_Fonts(display, gc, fontname, alt_font)
Display *display;
GC      gc;
char    *fontname;
char    *alt_font;
{
   XFontStruct *Load_Font();
   XFontStruct *font_info;
   char        *default_font = "vtbold";

   if (!(font_info = Load_Font(display,fontname))){
      fprintf(stderr,"Trying Alternate X11 font \"%s\"\n",alt_font);
      if (!(font_info = Load_Font(display,alt_font))){
         fprintf(stderr,"Trying Alternate X11 font \"%s\"\n",default_font);
         if (!(font_info = Load_Font(display,default_font))){
            fprintf(stderr,"***Error! Couldn't load any fonts!\n");
            return(NULL);
         }
      }
   }
   XSetFont(display, gc, font_info->fid);
   return(font_info);
}

/* Initialize and merge the various options */
Merge_Options()
{
   char    *option;

   /* process the various options */
   if (reverse < 0) {
      if ((option = XGetDefault(display,progname,"ReverseVideo")) != NULL)
         if (strcmp(option,"on")) reverse = 1;
   }
   if (!geometry) {
      option = XGetDefault(display,progname,"Geometry");
      geometry = option ? option : "=+5+5";
   }
   if (!font_name) {
      /* option = XGetDefault(display,progname,"BodyFont"); */
      font_name = DEFAULT_FONT;
   }
   if (!border_color) {
      option = XGetDefault(display,progname,"Border");
      border_color = option ? option : DEFAULT_BORDER_COLOR;
   }
   if (!back_color) {
      option = XGetDefault(display,progname,"Background");
      back_color = option ? option : DEFAULT_BACK_COLOR;
   }
   if (!fore_color) {
      option = XGetDefault(display,progname,"Foreground");
      fore_color = option ? option : DEFAULT_FORE_COLOR;
   }
   if (!mouse_color) {
      option = XGetDefault(display,progname,"Mouse");
      mouse_color = option ? option : DEFAULT_MOUSE_COLOR;
   }
   if (!icon_color) {
      option = XGetDefault(display,progname,"Icon");
      icon_color = option ? option : DEFAULT_ICON_COLOR;
   }
   if (border_width <= 0) {
      option = XGetDefault(display,progname,"BorderWidth");
      border_width = option ? atoi(option) : DEFAULT_BORDER_WIDTH;
   }
}

/* allocate the colors on color display */
int Alloc_Colors()
{
   XColor        cdef;
   Colormap      cmap;
   int           i, depth;
   unsigned long tmp_pixel;

   /* get info on depth and colormap */
   depth = DisplayPlanes(display,screen);
   cmap  = DefaultColormap(display,screen);

   /* associate named colors with pixel values */
   if (depth == 1) {       /* one-plane monochrome */
      colors[BDR_C]     = BlackPixel(display,screen);
      colors[BACK_C]    = BlackPixel(display,screen);
      colors[FORE_C]    = WhitePixel(display,screen);
      colors[MOUSE_C]   = colors[FORE_C];            
      colors[ICON_C]    = colors[FORE_C];            
      for (i=LINE1_C; i<=LINE5_C; i++) colors[i] = colors[FORE_C];
   } else {                /* color */
      clr_name[BDR_C]   = border_color;
      clr_name[BACK_C]  = back_color;
      clr_name[FORE_C]  = fore_color;
      clr_name[MOUSE_C] = mouse_color;
      clr_name[ICON_C]  = icon_color;
      clr_name[LINE1_C] = "yellow";
      clr_name[LINE2_C] = "cyan";
      clr_name[LINE3_C] = "green";
      clr_name[LINE4_C] = "red";
      clr_name[LINE5_C] = "blue";
      for (i=0; i<MAX_COLORS; i++) 
         if (XParseColor(display,cmap,clr_name[i],&cdef) &&
             XAllocColor(display,cmap,&cdef))
            colors[i] = cdef.pixel;
         else {
            fprintf(stderr,"\n%s: Can't allocate X11 color %s\n",
                    progname, clr_name[i]);
            return(0);
         } /* end if */
   }  /* end if */

   /* assign pixel values */
   border_pixel      = colors[BDR_C];
   background_pixel  = colors[BACK_C];
   foreground_pixel  = colors[FORE_C];
   mouseground_pixel = colors[MOUSE_C];
   icon_pixel        = colors[ICON_C];

   /* reverse video */
   if (reverse) {
      tmp_pixel        = background_pixel;
      background_pixel = foreground_pixel;
      foreground_pixel = tmp_pixel;
      if (border_pixel == background_pixel)
         border_pixel = foreground_pixel;
      if (mouseground_pixel == background_pixel) 
         mouseground_pixel = foreground_pixel;
      for (i=LINE1_C; i<=LINE5_C; i++)
         if (colors[i]==background_pixel) colors[i] = foreground_pixel;
   }

   /* return */
   return(1);
}

RunOps()
{
#  define   SMALLW 1
#  define   OK     0
   XEvent   event;
   KeySym   keysym;
   XComposeStatus compose;
   char     buffer[1];
   int      bufsize = 1;
   int      window_size = OK;
   int      Width, Height;       

   /* Initialize window size */
   Width  = width;
   Height = height;

   /* event loop */
   while(1) {
      XNextEvent(display, &event);
      switch (event.type) {
      case Expose:
         /* get rid of all other Expose events on the queue */
         while (XCheckTypedEvent(display, Expose, &event));
         /* window was resized too small to use */
         if (window_size == SMALLW) {
            Width  = Xxdim;
            Height = Xydim;
         }
         ShowView(Width, Height);
         break;
      case ConfigureNotify:
         /* window has been resized */
         /* change width and height */
         Width  = event.xconfigure.width;
         Height = event.xconfigure.height;
         if ((Width < size_hints.min_width) ||
             (Height < size_hints.min_height) ) 
            window_size = SMALLW;
         else 
            window_size = OK;
         break;
      case KeyPress:
         XLookupString((XKeyEvent *)&event,buffer,bufsize,&keysym,&compose);
         if ((keysym == XK_q) || (keysym == XK_Q)) {
            XUnloadFont(display, font_info->fid);
            XUnloadFont(display, lblfont_info->fid);
            XFreeGC(display, gc);
            XFreeGC(display, gcl);
            XCloseDisplay(display);
            return;
         }
         break;
      case ButtonPress:
         XUnloadFont(display, font_info->fid);
         XUnloadFont(display, lblfont_info->fid);
         XFreeGC(display, gc);
         XFreeGC(display, gcl);
         XCloseDisplay(display);
         return;
         break;
      default:
         break;
      } /* end switch */
   } /* end while */
}

ShowView(Width, Height)
int      Width, Height;       
{
   axesX(Width, Height);
   plotX();
}

/* draw the axes */
axesX(Width, Height)
int   Width, Height;       
{
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     equalscale;
   extern int     autorange, xautorange, yautorange;
   extern double  xmin, xmax, ymin, ymax;

   int    Xxwid, Xywid;
   double ratio;
   char   text[100], c, time[MAXCHAR];
   int    text_len, text_width, text_xpos, text_ypos;
   int    font_height, font_width, label_len;
   int    i;

   /* Initialize */
   Xxmin = DEFAULT_BDR_DIM;
   Xymin = DEFAULT_BDR_DIM;
   Xxwid = Width  - 2*DEFAULT_BDR_DIM;
   Xywid = Height - 2*DEFAULT_BDR_DIM;

   if (equalscale == ON) {
      /* Fit the plot inside the window */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio < ((double)Xywid)/((double)Xxwid))
         Xywid  = Xxwid*ratio;
      else
         Xxwid  = Xywid/ratio;
   } else {
      /* The plot must fit inside the window */
      if (Xxwid*xyratio < Xywid)
         Xywid = Xxwid*xyratio;
      else
         Xxwid = Xywid/xyratio;
   }
   Xxmax = Xxmin + Xxwid;
   Xymax = Xymin + Xywid;


   /* Draw the main axes */
   XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
   XDrawRectangle(display,window,gc,Xxmin,Xymin,Xxwid,Xywid);

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* Top Label */
   text_len   = strlen(toplabel);
   text_width = XTextWidth(font_info,toplabel,text_len);
   text_xpos  = (Xxmin + Xxmax - text_width)/2;
   text_ypos  = Xymin - 3*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,toplabel,text_len);

   /* Draw the tick marks and labels */
   label_len = 6;
   if (!autorange) {
      plot_Xxticks(&label_len);
      plot_Xyticks(&label_len);
   } else {
      if (xautorange && !xlog) plot_XAxticks(&label_len);
      else                     plot_Xxticks(&label_len);
      if (yautorange && !ylog) plot_XAyticks(&label_len);
      else                     plot_Xyticks(&label_len);
   }

   /* X-Axis Label */
   text_len   = strlen(xlabel);
   text_width = XTextWidth(font_info,xlabel,text_len);
   text_xpos  = (Xxmin + Xxmax - text_width)/2;
   text_ypos  = Xymax + 4*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,xlabel,text_len);

   /* Y-Axis Label */
   /* Don't know how to rotate, so print character by character */
   text_len   = strlen(ylabel);
   text_xpos  = Xxmin - (label_len+6)*font_width;
   text_ypos  = (Xymin + Xymax)/2 + 0.5*font_height
                - 0.50*(text_len-1)*((int)(1.2*font_height));
   for (i=0; (c=ylabel[i]) != '\0'; i++) {
      text[0] = c;
      XDrawString(display,window,gc,text_xpos,text_ypos,text,1);
      text_ypos += (int)(1.2*font_height);
   }

   /* side labels */
   sprintf(text,"%s","Minimum");
   text_len  = strlen(text);
   text_width= XTextWidth(font_info,text,text_len);
   text_xpos = Xxmax + 40 - text_width/2;
   text_ypos = (Xymin + Xymax)/2 - 6*font_height;  
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
   sprintf(text,"%.2g",zmin);
   text_len  = strlen(text);
   text_width= XTextWidth(font_info,text,text_len);
   text_xpos = Xxmax + 40 - text_width/2;
   text_ypos = (Xymin + Xymax)/2 - 4*font_height;  
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);

   /* side labels */
   sprintf(text,"%s","Maximum");
   text_len  = strlen(text);
   text_width= XTextWidth(font_info,text,text_len);
   text_xpos = Xxmax + 40 - text_width/2;
   text_ypos = (Xymin + Xymax)/2 + 4*font_height;  
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
   sprintf(text,"%.2g",zmax);
   text_len  = strlen(text);
   text_width= XTextWidth(font_info,text,text_len);
   text_xpos = Xxmax + 40 - text_width/2;
   text_ypos = (Xymin + Xymax)/2 + 6*font_height;  
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);

   /* put on a time label */
   CNget_localtime(time,MAXCHAR);
   text_len   = strlen(time);
   text_width = XTextWidth(font_info,time,text_len);
   text_xpos  = 10 ;
   text_ypos  = 1.2*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,time,text_len);

}

/* Draw the x-axis tick marks and labels */
plot_Xxticks(label_len)
int  *label_len;
{
   extern int     grid,xticks,xlog;
   extern double  xmin, xmax;

   double xdx, xintv;
   double vallbl;
   int    Xxtmp;
   int    font_height, font_width;
   int    i, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   } 

   /* useful data */
   xintv = (Xxmax-Xxmin)/(double)xticks;
   xdx   = (xmax-xmin)/(double)xticks;

   /* xticks */
   for (i=0; i<=xticks; i++) {
      Xxtmp = Xxmin + (int)(i*xintv);
      XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+10);
      XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-10);
      vallbl = xmin +i*xdx;
      plot_Xxticklabel(vallbl,Xxtmp,font_height,font_width);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
      XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
      for (i=1; i<xticks; i++) {
         Xxtmp = Xxmin + (int)(i*xintv);
         XDrawLine(display,window,gc,Xxtmp,Xymin+10,Xxtmp,Xymax-10);
      }
      XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
   }
}

/* Draw the y-axis tick marks and labels */
plot_Xyticks(label_len)
int  *label_len;
{
   extern int     grid,yticks,ylog;
   extern double  ymin, ymax;

   double ydy, yintv;
   double vallbl;
   int    Xytmp;
   int    font_height, font_width;
   int    i, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* reset the number of ticks */
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-25) + 1;
      ymax   = ymin + yticks;
   } 

   /* useful data */
   yintv = (Xymax-Xymin)/(double)yticks;
   ydy   = (ymax-ymin)/(double)yticks;

   /* yticks */
   for (i=0; i<=yticks; i++) {
      Xytmp = Xymin + (int)(i*yintv);
      XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+10,Xytmp);
      XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-10,Xytmp);
      vallbl = ymax -i*ydy;
      plot_Xyticklabel(vallbl,Xytmp,font_height,font_width,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
      XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
      for (i=1; i<yticks; i++) {
         Xytmp = Xymin + (int)(i*yintv);
         XDrawLine(display,window,gc,Xxmin+10,Xytmp,Xxmax-10,Xytmp);
      }
      XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
   }
}

/* Draw the x-axis tick marks and labels with automatic ranging */
plot_XAxticks(label_len)
int  *label_len;
{
   extern int     grid,xticks,xlog;
   extern double  xmin, xmax;

   double xmin2, xmax2, deltx;
   double xtmp, xt, xfactor;
   int    Xxtmp;
   int    font_height, font_width;
   int    i, nticks, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* 
    * useful data 
    * X = Xxmin + (x-xmin)*(Xxmax-Xxmin)/(xmax-xmin) 
    * Y = Xymax + (y-ymin)*(Xymin-Xymax)/(ymax-ymin) 
    */
   xfactor = (Xxmax - Xxmin)/(xmax-xmin);
   
   /* get rounded limits */
   CNget_autorange(xmin,xmax,&xmin2,&xmax2,&deltx);

   /* xticks */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i); 
         if ((xt > xmin) && (xt < xmax)) {
            Xxtmp = Xxmin + (xt-xmin)*xfactor;
            /* Draw the grid */
            if (grid == ON) {
               XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
               XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
               XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymax);
               XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
            }
            /* Draw the ticks */
            if (i==5) {
               XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+7);
               XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-7);
            } else {
               XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+4);
               XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-4);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) { 
      Xxtmp = Xxmin + (xtmp-xmin)*xfactor;
      /* Draw the grid */
      if (grid == ON) {
         XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
         XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
         XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymax);
         XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
      }
      /* Draw the ticks */
      XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+10);
      XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-10);
      plot_Xxticklabel(xtmp,Xxtmp,font_height,font_width);
      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   /* 
    * the first label is at xmin2 + deltx
    * want to put on a label at xmin < xmin2 + delx,
    * but this label should not overlap with the first label
    * Thus, xmin2 + deltx - xmin > 0.1 (xmax - xmin)
    *     (0th tick to 1st tick)   (full size)
    */
   if ((nticks==0) || ((xmin2 + deltx - xmin) > 0.1*(xmax-xmin))) {
      plot_Xxticklabel(xmin,Xxmin,font_height,font_width);
   }

   /* put labels on the min and max points */
   /* 
    * the last label is at xtmp - deltx
    * want to put on a label at xmax > xtmp - deltx
    * but this label should not overlap with the last label
    * Thus, xmax - (xtmp - deltx) > 0.1 (xmax - xmin)
    *     (0th tick to 1st tick)   (full size)
    */
   if ((nticks==0) || ((xmax - xtmp + deltx) > 0.1*(xmax-xmin))) {
      plot_Xxticklabel(xmax,Xxmax,font_height,font_width);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;
}

/* Draw the y-axis tick marks and labels with automatic ranging */
plot_XAyticks(label_len)
int  *label_len;
{
   extern int     grid,yticks,ylog;
   extern double  ymin, ymax;

   double ymin2, ymax2, delty;
   double ytmp, yt, yfactor;
   int    Xytmp;
   int    font_height, font_width;
   int    i, nticks, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* 
    * useful data 
    * X = Xxmin + (x-xmin)*(Xxmax-Xxmin)/(xmax-xmin) 
    * Y = Xymax + (y-ymin)*(Xymin-Xymax)/(ymax-ymin) 
    */
   yfactor = (Xymin - Xymax)/(ymax-ymin);
   
   /* get rounded limits */
   CNget_autorange(ymin,ymax,&ymin2,&ymax2,&delty);

   /* yticks */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i); 
         if ((yt > ymin) && (yt < ymax)) {
            Xytmp = Xymax + (yt-ymin)*yfactor;
            /* Draw the grid */
            if (grid == ON) {
               XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
               XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
               XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmax,Xytmp);
               XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
            }
            /* Draw the ticks */
            if (i==5) {
               XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+7,Xytmp);
               XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-7,Xytmp);
            } else {
               XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+4,Xytmp);
               XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-4,Xytmp);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) { 
      Xytmp = Xymax + (ytmp-ymin)*yfactor;
      /* Draw the grid */
      if (grid == ON) {
         XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);
         XSetDashes(display,gc,1,(char *)dash_list_long_dotted,LONG_DOTTED);
         XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmax,Xytmp);
         XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
      }
      /* Draw the ticks */
      XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+10,Xytmp);
      XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-10,Xytmp);
      plot_Xyticklabel(ytmp,Xytmp,font_height,font_width,&llen);
      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   /*
    * the first label is at ymin2 + delty
    * want to put on a label at ymin < ymin2 + dely,
    * but this label should not overlap with the first label
    * Thus, ymin2 + delty - ymin > 0.1 (ymax - ymin)
    *     (0th tick to 1st tick)   (full size)
    */
   if ((nticks==0) || ((ymin2 + delty - ymin) > 0.1*(ymax-ymin))) {
      plot_Xyticklabel(ymin,Xymax,font_height,font_width,&llen);
   }

   /* put labels on the min and max points */
   /* 
    * the last label is at ytmp - delty
    * want to put on a label at ymax > ytmp - delty
    * but this label should not overlap with the last label
    * Thus, ymax - (ytmp - delty) > 0.1 (ymax - ymin)
    *     (0th tick to 1st tick)   (full size)
    */
   if ((nticks==0) || ((ymax - ytmp + delty) > 0.1*(ymax-ymin))) {
      plot_Xyticklabel(ymax,Xymin,font_height,font_width,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;
}

/* plot the tick label on the x-axis */
/*ARGSUSED*/
plot_Xxticklabel(vallbl,Xxtmp,font_height,font_width)
double vallbl;
int    Xxtmp, font_height, font_width;
{
   extern int xlog;
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   text_width = XTextWidth(font_info,text,text_len);
   text_xpos  = Xxtmp  - text_width/2;
   text_ypos  = Xymax + 2*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
}

/* plot the tick label on the y-axis */
/*ARGSUSED*/
plot_Xyticklabel(vallbl,Xytmp,font_height,font_width,llen)
double vallbl;
int    Xytmp, font_height, font_width, *llen;
{
   extern int ylog;
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   if (text_len > *llen) *llen = text_len;
   text_width = XTextWidth(font_info,text,text_len);
   text_xpos  = Xxmin - text_width - font_width;
   text_ypos  = Xytmp  + font_height/2;
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
}

/* draw the plot */
plotX()
{
   extern int     clip, contlabel;
   extern double  xmin, xmax, ymin, ymax;
   extern CNplotsetptr cont_plothead;

   CNplotsetptr P;
   CNcurveptr   C;
   CNnodeptr    N;

   XRectangle  rectangles[1];
   XPoint      points[MAXPTS];
   double      idx,jdy;
   int         text_len, text_width, text_xpos, text_ypos, font_height;
   int         npts, j;

   /* clip henceforth */
   if (clip) {
      rectangles[0].x      = Xxmin;
      rectangles[0].y      = Xymin;
      rectangles[0].width  = Xxmax - Xxmin;
      rectangles[0].height = Xymax - Xymin;
      XSetClipRectangles(display,gc,1,0,rectangles,1,Unsorted);
   }

   /* initialize */
   font_height = lblfont_info->max_bounds.ascent +
                 lblfont_info->max_bounds.descent;
   idx = (Xxmax-Xxmin)/(xmax-xmin);
   jdy = (Xymax-Xymin)/(ymin-ymax);  /* because of strange x format */

   /* Plot set - covers multiple plots */
   for (P=cont_plothead; P!=NULL; P=P->next) {

      /* Curve set */
      for (C=P->curvehead; C!=NULL; C=C->next) {

         /* First node */
         N = C->nodehead;

         /* go thru loop until we run out of points */
         while (N != NULL) {
            j = 0;

            /* rescale points, save in array */
            for ( ; N!=NULL && j<MAXPTS; N=N->next) {
               points[j].x = (int)((N->pt.x-xmin)*idx) + Xxmin;
               points[j].y = (int)((N->pt.y-ymax)*jdy) + Xymin;
               j++;
            }
            npts = j;

            if (npts>0) {
               /* plot the points */
               linetypX(C->linetype,C->linewidth);
               linecolX(C->linetype);
               XDrawLines(display,window,gc,points,npts,CoordModeOrigin);
               XSetForeground(display,gc,foreground_pixel); 

               /* label the contours */
               if ((P->plottype == CN_CONTOUR) && contlabel && 
                   (C->linetype == CN_LNSOLID || C->linetype == CN_LNDASHED)){
                  text_len   = strlen(C->label);
                  text_width = XTextWidth(lblfont_info,C->label,text_len);
                  text_xpos  = points[(int)(0.5*j)].x - text_width/2;
                  text_ypos  = points[(int)(0.5*j)].y - font_height/2;
                  XDrawString(display,window,gcl,
                              text_xpos,text_ypos,C->label,text_len);
               }
            }

            /* if there are more points to come, repeat the last point */
            if (N != NULL) N = N->prev;
         }
      }
   }
   /* stop clipping */
   if (clip) {
      XSetClipMask(display,gc,None);
   }
}

/* set the line type and linewidth*/
int linetypX(linetyp, ithk)
int linetyp;
int ithk;
{
   /* Reset the linewidth if necessary */
   if (ithk <= 0) ithk = 1;
   linetyp = linetyp % 11;

#ifdef DEC3100
   linetyp = linetyp % 1;
#endif /* DEC3100 */

   switch (linetyp) {
   case 0 : XSetLineAttributes(display,gc,ithk,LineSolid,CapButt,JoinBevel);
            break;
   case 1 : XSetLineAttributes(display,gc,ithk,LineSolid,CapButt,JoinBevel);
            break;
   case 2 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_dashed,DASHED);
            break;
   case 3 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_dotted,DOTTED);
            break;
   case 4 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_dot_dashed,DOT_DASHED);
            break;
   case 5 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_short_dotted,SHORT_DOTTED);
            break;
   case 6 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_short_dashed,SHORT_DASHED);
            break;
   case 7 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_odd_dashed,ODD_DASHED);
            break;
   case 8 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_long_dashed,LONG_DASHED);
            break;
   case 9 : XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_dash_dotted,DASH_DOTTED);
            break;
   case 10: XSetLineAttributes(display,gc,ithk,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,ithk,(char *)dash_list_equal_dashed,EQUAL_DASHED);
            break;
   default: XSetLineAttributes(display,gc,ithk,LineSolid,CapButt,JoinBevel);
            break;
   }
}

/* choose a linecolor */
linecolX(linetyp)
int linetyp;
{
   linetyp = linetyp % 11;
   switch (linetyp) {
      case 0 : XSetForeground(display,gc,foreground_pixel); break;
      case 1 : XSetForeground(display,gc,colors[LINE1_C]); break;
      case 2 : XSetForeground(display,gc,colors[LINE2_C]); break;
      case 3 : XSetForeground(display,gc,colors[LINE3_C]); break;
      case 4 : XSetForeground(display,gc,colors[LINE4_C]); break;
      case 5 : XSetForeground(display,gc,colors[LINE5_C]); break;
      case 6 : XSetForeground(display,gc,colors[LINE1_C]); break;
      case 7 : XSetForeground(display,gc,colors[LINE2_C]); break;
      case 8 : XSetForeground(display,gc,colors[LINE3_C]); break;
      case 9 : XSetForeground(display,gc,colors[LINE4_C]); break;
      case 10: XSetForeground(display,gc,colors[LINE5_C]); break;
      default: XSetForeground(display,gc,foreground_pixel); break;
   }
}


#endif  /* NO_X11PLOT */
