/*
 * plotps.c - POSTSCRIPT subroutines for drawing plots
 */

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"

#define PX_DIM   600
#define PY_DIM   800
#define PXL_DIM  800
#define PYL_DIM  600
#define PBDR_DIM 100

double  Pxmin, Pymin, Pxmax, Pymax;

/* Output Postscript Subroutines */
initps(ips)
FILE *ips;
{
   extern double scale;
   char   time[MAXCHAR];

   /* begin postscript output */
   fprintf(ips,"%c!\n",'%');

   if (landscape) fprintf(ips,"-90 rotate -790 0 translate\n");

   fprintf(ips,"/defFontSizeTop %6.2f def\n",25*scale);
   fprintf(ips,"/defFontSizeSide %6.2f def\n",20*scale);
   fprintf(ips,"/defFontSizeAxes %6.2f def\n",15*scale);
   fprintf(ips,"/defFontSizeMkrs %6.2f def\n",12*scale);
   fprintf(ips,"/defFontSizeLbls %6.2f def\n",8*scale);

   fprintf(ips,"/UPJ  {} def\n");
   fprintf(ips,"/MIJT  {0 defFontSizeTop -2 div rmoveto} def\n");
   fprintf(ips,"/BOJT  {0 defFontSizeTop neg rmoveto} def\n");
   fprintf(ips,"/MIJS  {0 defFontSizeSide -2 div rmoveto} def\n");
   fprintf(ips,"/BOJS  {0 defFontSizeSide neg rmoveto} def\n");
   fprintf(ips,"/MIJA  {0 defFontSizeAxes -2 div rmoveto} def\n");
   fprintf(ips,"/BOJA  {0 defFontSizeAxes neg rmoveto} def\n");
   fprintf(ips,"/MIJM  {0 defFontSizeLbls -2 div rmoveto} def\n");
   fprintf(ips,"/BOJM  {0 defFontSizeLbls neg rmoveto} def\n");
   fprintf(ips,"/MIJL  {0 defFontSizeMkrs -3 div rmoveto} def\n");
   fprintf(ips,"/BOJL  {0 defFontSizeMkrs neg rmoveto} def\n");

   fprintf(ips,"/LEJ  {} def\n");
   fprintf(ips,"/CEJ  {dup stringwidth pop -2 div 0 rmoveto} def\n");
   fprintf(ips,"/RIJ  {dup stringwidth pop neg 0 rmoveto} def\n");

   /* print the time */
   CNget_localtime(time,MAXCHAR);
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"10 scalefont setfont\n");
   if (landscape)
      fprintf(ips,"%6.2f %6.2f moveto ",30.0,570.0);
   else 
      fprintf(ips,"%6.2f %6.2f moveto ",30.0,750.0);
   fprintf(ips,"(%s) show\n",time);
}

/* draw the axes */
axesps(ips)
FILE *ips;
{
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     equalscale, clip;
   extern int     linechange, markerchange;
   extern int     autorange, xautorange, yautorange;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xwid,ywid,ratio;
   int    label_len,dist;
   char   label[MAXCHAR];

   /* Initialize */
   Pxmin = PBDR_DIM;
   Pymin = PBDR_DIM;
   xwid  = PX_DIM - 2*PBDR_DIM;          /* 400 */
   ywid  = PY_DIM - 2*PBDR_DIM;          /* 600 */
   if (landscape) {
      xwid  = PXL_DIM - 2*PBDR_DIM;      /* 600 */
      ywid  = PYL_DIM - 2*PBDR_DIM;      /* 400 */
   }
   Pxmax = Pxmin + scale*xwid;
   Pymax = Pymin + scale*ywid; 

   if (equalscale == ON) {
      /* Fit the plot inside the window */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio <= (ywid/xwid)) {
         /* decrease ywid : ywid  = xwid*ratio */
         Pymax = Pymax - scale*0.5*(ywid - xwid*ratio);
         Pymin = Pymin + scale*0.5*(ywid - xwid*ratio);
      } else {
         /* decrease xwid : xwid  = ywid/ratio */
         Pxmax = Pxmax - scale*0.5*(xwid - ywid/ratio);
         Pxmin = Pxmin + scale*0.5*(xwid - ywid/ratio);
         Pxmax = Pxmin + scale*ywid/ratio;
      }
   } else {
      /* The plot must fit inside the window */
      if (xwid*xyratio < ywid) {
         /* decrease ywid : ywid  = xwid*xyratio */
         Pymax = Pymax - scale*0.5*(ywid - xwid*xyratio);
         Pymin = Pymin + scale*0.5*(ywid - xwid*xyratio);
      } else {
         /* decrease xwid : xwid  = ywid/ratio */
         Pxmax = Pxmax - scale*0.5*(xwid - ywid/xyratio);
         Pxmin = Pxmin + scale*0.5*(xwid - ywid/xyratio);
         Pxmax = Pxmin + scale*ywid/xyratio;
      }
   }


   /* define the axes */
   fprintf(ips,"/rectanglepath {\n");
   fprintf(ips,"newpath\n  %6.2f %6.2f moveto\n",Pxmin,Pymin);
   fprintf(ips,"  0.0 %6.2f rlineto\n",Pymax-Pymin);
   fprintf(ips,"  %6.2f 0.0 rlineto\n",Pxmax-Pxmin);
   fprintf(ips,"  0.0 %6.2f rlineto\n",Pymin-Pymax);
   fprintf(ips,"closepath } def\n");
   fprintf(ips,"%6.2f setlinewidth\nrectanglepath stroke\n",2*scale);

   /* Top Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeTop scalefont setfont\n");
   fprintf(ips,"  %6.2f %6.2f moveto ",0.5*(Pxmax+Pxmin),Pymax+25*scale);
   fprintf(ips,"(%s) UPJ CEJ show\n",toplabel);

   /* Draw the tick marks and labels */
   fprintf(ips,"%6.2f setlinewidth\n",1*scale);
   label_len = 6;
   if (!autorange) {
      plot_PSxticks(ips);
      plot_PSyticks(ips,&label_len);
   } else {
      if (xautorange && !xlog) plot_PSAxticks(ips);
      else                     plot_PSxticks(ips);
      if (yautorange && !ylog) plot_PSAyticks(ips,&label_len);
      else                     plot_PSyticks(ips,&label_len);
   }

   /* X-axis Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeSide scalefont setfont\n");
   fprintf(ips,"  %6.2f %6.2f moveto ",0.5*(Pxmax+Pxmin),Pymin-20*scale);
   fprintf(ips,"(%s) BOJS CEJ show\n",xlabel);

   /* Y-axis Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeSide scalefont setfont\n");
   fprintf(ips,"/Ylabel {");
   dist = 10*label_len + 5;
   if (dist > 65) dist = 65;
   fprintf(ips,"%6.2f %6.2f moveto ",0.5*(Pymax+Pymin),
           dist*scale-Pxmin);
   fprintf(ips,"(%s) UPJ CEJ show ",ylabel);
   fprintf(ips,"} def\n  90 rotate\n  Ylabel\n  -90 rotate\n"); 

   /* side labels */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeMkrs scalefont setfont\n");
   sprintf(label,"%s","MINIMUM");
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmax+40*scale,Pymax-20*scale);
   fprintf(ips,"  (%s) MIJM CEJ show\n",label);
   sprintf(label,"%.2g",zmin);
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmax+40*scale,Pymax-40*scale);
   fprintf(ips,"  (%s) MIJM CEJ show\n",label);
   sprintf(label,"%s","MAXIMUM");
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmax+40*scale,Pymin+40*scale);
   fprintf(ips,"  (%s) MIJM CEJ show\n",label);
   sprintf(label,"%.2g",zmax);
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmax+40*scale,Pymin+20*scale);
   fprintf(ips,"  (%s) MIJM CEJ show\n",label);

   /* clip if necessary */
   fprintf(ips,"%6.2f setlinewidth\n",0.5*scale);
   if (clip) fprintf(ips,"rectanglepath clip\n");
}

/* Draw the x-axis tick marks and labels */
plot_PSxticks(ips)
FILE *ips;
{
   extern int     grid,xticks,xlog;
   extern double  scale;
   extern double  xmin, xmax;

   double xdx,xintv;
   double Pxtmp,Pytmp;
   double vallbl;
   int    i;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   }

   /* Useful data */
   xdx   = (xmax-xmin)/xticks;
   xintv = (Pxmax-Pxmin)/(double)xticks;

   /* Tick Marks on the X-axis */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeAxes scalefont setfont\n");
   for (i=0; i<=xticks; i++) {
      Pxtmp = Pxmin + xintv*i;
      Pytmp = Pymin;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",7*scale);

      vallbl = xmin + i*xdx;
      plot_PSxticklabel(ips,vallbl,Pxtmp);

      Pytmp = Pymax;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",-7*scale);
   }

   /* draw the grid */
   if (grid == ON) {
      for (i=1; i<xticks; i++) {
         Pxtmp = Pxmin + xintv*i;
         Pytmp = Pymax - 8*scale;
         fprintf(ips,"[] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"0 %6.2f rlineto stroke\n",Pymin-Pymax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
         linetypps(ips,1);
      }
   }
}

/* Draw the y-axis tick marks and labels */
plot_PSyticks(ips,label_len)
FILE *ips;
int  *label_len;
{
   extern int     grid,yticks,ylog;
   extern double  scale;
   extern double  ymin, ymax;

   double ydy,yintv;
   double Pxtmp,Pytmp;
   double vallbl;
   int    i, llen = 0;

   /* reset the number of ticks */
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   }

   /* Useful data */
   ydy   = (ymax-ymin)/yticks;
   yintv = (Pymax-Pymin)/(double)yticks;

   /* Tick Marks on the Y-axis */
   for (i=0; i<=yticks; i++) {
      Pytmp = Pymin + yintv*i;
      Pxtmp = Pxmin;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",7*scale);

      vallbl = ymin + i*ydy;
      plot_PSyticklabel(ips,vallbl,Pytmp,&llen);

      Pxtmp = Pxmax;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",-7*scale);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* draw the grid */
   if (grid == ON) {
      for (i=1; i<yticks; i++) {
         Pytmp = Pymin + yintv*i;
         Pxtmp = Pxmax - 8*scale;
         fprintf(ips,"[] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"%6.2f 0 rlineto stroke\n",Pxmin-Pxmax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
         linetypps(ips,1);
      }
   }
}

/* Draw the x-axis tick marks and labels with automatic ranging */
plot_PSAxticks(ips)
FILE *ips;
{
   extern int     grid,xticks,xlog;
   extern double  scale;
   extern double  xmin, xmax;

   double xmin2, xmax2, deltx;
   double xtmp, xt, xfactor;
   double Pxtmp, Pytmp;
   int    i, nticks;

   /*
    * useful data
    * X = Pxmin + (x-xmin)*(Pxmax-Pxmin)/(xmax-xmin)
    * Y = Pymin + (y-ymin)*(Pymax-Pymin)/(ymax-ymin)
    */
   xfactor = (Pxmax - Pxmin)/(xmax-xmin);

   /* get rounded limits */
   CNget_autorange(xmin,xmax,&xmin2,&xmax2,&deltx);

   /* Tick Marks on the X-axis */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i);
         if ((xt > xmin) && (xt < xmax)) {
            Pxtmp = Pxmin + (xt-xmin)*xfactor;
            /* Draw the grid */
            if (grid == ON) {
               fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pymin);
               fprintf(ips,"0 %6.2f rlineto stroke\n",Pymax-Pymin);
               fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
            }
            /* Draw the tickmarks */
            if (i==5) {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
               fprintf(ips,"  0 %6.2f rlineto stroke\n", 5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
               fprintf(ips,"  0 %6.2f rlineto stroke\n",-5*scale);
            } else {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
               fprintf(ips,"  0 %6.2f rlineto stroke\n", 3*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
               fprintf(ips,"  0 %6.2f rlineto stroke\n",-3*scale);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeAxes scalefont setfont\n");
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) {
      Pxtmp = Pxmin + (xtmp-xmin)*xfactor;

      /* Draw the grid */
      if (grid == ON) {
         fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pymin);
         fprintf(ips,"0 %6.2f rlineto stroke\n",Pymax-Pymin);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
      }

      /* Draw the tickmarks */
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
      fprintf(ips,"  0 %6.2f rlineto stroke\n", 7*scale);
      plot_PSxticklabel(ips,xtmp,Pxtmp);
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",-7*scale);

      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin2 + deltx - xmin) > 0.1*(xmax-xmin))) {
      plot_PSxticklabel(ips,xmin,Pxmin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmax - xtmp + deltx) > 0.1*(xmax-xmin))) {
      plot_PSxticklabel(ips,xmax,Pxmax);
   }
}

/* Draw the y-axis tick marks and labels with automatic ranging */
plot_PSAyticks(ips,label_len)
FILE *ips;
int  *label_len;
{
   extern int     grid,yticks,ylog;
   extern double  scale;
   extern double  ymin, ymax;

   double ymin2, ymax2, delty;
   double ytmp, yt, yfactor;
   double Pxtmp, Pytmp;
   int    i, nticks, llen = 0;

   /*
    * useful data
    * X = Pxmin + (x-xmin)*(Pxmax-Pxmin)/(xmax-xmin)
    * Y = Pymin + (y-ymin)*(Pymax-Pymin)/(ymax-ymin)
    */
   yfactor = (Pymax - Pymin)/(ymax-ymin);

   /* get rounded limits */
   CNget_autorange(ymin,ymax,&ymin2,&ymax2,&delty);

   /* Tick Marks on the Y-axis */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i);
         if ((yt > ymin) && (yt < ymax)) {
            Pytmp = Pymin + (yt-ymin)*yfactor;
            /* Draw the grid */
            if (grid == ON) {
               fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxmin,Pytmp);
               fprintf(ips,"%6.2f 0 rlineto stroke\n",Pxmax-Pxmin);
               fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
            }
            /* Draw the tickmarks */
            if (i==5) {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n", 5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n",-5*scale);
            } else {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n", 3*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n",-3*scale);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) {
      Pytmp = Pymin + (ytmp-ymin)*yfactor;

      /* Draw the grid */
      if (grid == ON) {
         fprintf(ips,"%6.2f setlinewidth 0.9 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxmin,Pytmp);
         fprintf(ips,"%6.2f 0 rlineto stroke\n",Pxmax-Pxmin);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",1.0*scale);
      }

      /* Draw the tickmarks */
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n", 7*scale);
      plot_PSyticklabel(ips,ytmp,Pytmp,&llen);
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",-7*scale);

      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin2 + delty - ymin) > 0.1*(ymax-ymin))) {
      plot_PSyticklabel(ips,ymin,Pymin,&llen);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymax - ytmp + delty) > 0.1*(ymax-ymin))) {
      plot_PSyticklabel(ips,ymax,Pymax,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;
}

/* plot the tick label on the x-axis */
plot_PSxticklabel(ips,vallbl,Pxtmp)
FILE *ips;
double vallbl, Pxtmp;
{
   extern int     xlog;

   char   label[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   fprintf(ips,"  %6.2f %6.2f moveto",Pxtmp,Pymin);
   fprintf(ips,"  (%s) BOJA CEJ show\n",label);
}

/* plot the tick label on the y-axis */
plot_PSyticklabel(ips,vallbl,Pytmp,llen)
FILE *ips;
double vallbl, Pytmp;
int    *llen;
{
   extern int     xlog;

   int    label_len;
   char   label[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   label_len = strlen(label);
   if (label_len > *llen) *llen = label_len;
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmin-5*scale,Pytmp+3*scale);
   fprintf(ips,"  (%s) MIJA RIJ show\n",label);
}

/* draw the plot */
plotps(ips)
FILE *ips;
{
   int    set_label_pt();
   extern int     contlabel;
   extern double  xmin, xmax, ymin, ymax;

   extern CNplotsetptr cont_plothead;
   CNplotsetptr P;
   CNcurveptr   C;
   CNnodeptr    N;
   double       x,y,xn, xm, yn, ym, idx, jdy, angle;
   int          npts, j, m, n;

   /* initialize */
   idx = (Pxmax-Pxmin)/(xmax-xmin);
   jdy = (Pymax-Pymin)/(ymax-ymin);

   /* Plot set - covers multiple plots */
   for (P=cont_plothead; P!=NULL; P=P->next) {

      /* Curve set */
      for (C=P->curvehead; C!=NULL; C=C->next) {

         /* first count the number of points in the curve */
         npts=0;
         for (N=C->nodehead; N!=NULL; N=N->next) npts++;
         m = set_label_pt(npts);
         n = m+1;

         if (npts>0) {
            /* plot the points */
            linetypps(ips,C->linetype);
            N = C->nodehead;
            x = (N->pt.x-xmin)*idx + Pxmin;
            y = (N->pt.y-ymin)*jdy + Pymin;
            fprintf(ips,"newpath\n  %8.2f %8.2f moveto\n",x,y);
            j = 1;
            N = N->next;
            for ( ; N!=NULL; N=N->next) {
               x = (N->pt.x-xmin)*idx + Pxmin;
               y = (N->pt.y-ymin)*jdy + Pymin;
               fprintf(ips,"  %8.2f %8.2f lineto\n",x,y);
               if (j==m) {xm = x; ym = y;}
               if (j==n) {xn = x; yn = y;}
               j++;
            } 
            fprintf(ips,"stroke\n");

            /* now put on the labels */
            if ((P->plottype == CN_CONTOUR) && contlabel && 
               (C->linetype == CN_LNSOLID || C->linetype == CN_LNDASHED) &&
               (m>=0 && n<npts)) {
               if (fabs(xm-xn)< 1.0e-5)
                  angle = -90.0;
               else
                  angle = 360.0/(2*3.14159)*atan((yn-ym)/(xn-xm));
               labelps(ips,xm,ym,angle,C->label);
            }
         }
      }
   }
}

/* find the linetype */
linetypps(ips,line)
FILE *ips;
int line;
{
   line = line % 11;

   switch (line) {
   case 0 : fprintf(ips,"[0 1] 0 setdash\n"); break;    /* No Line */
   case 1 : fprintf(ips,"[] 0 setdash\n"); break;       /* Solid */
   case 2 : fprintf(ips,"[6 6] 0 setdash\n"); break;    /* Dashed */
   case 3 : fprintf(ips,"[1 2] 1 setdash\n"); break;    /* Dotted */
   case 4 : fprintf(ips,"[6 2 2 2] 0 setdash\n"); break;/* Dot Dash */
   case 5 : fprintf(ips,"[2 2 2 6] 0 setdash\n"); break;/* Double Dot */
   case 6 : fprintf(ips,"[8 2] 0 setdash\n"); break;    /* Long Dash */
   case 7 : fprintf(ips,"[9 6 2 6] 0 setdash\n"); break;/* Dot Dash */
   case 8 : fprintf(ips,"[2 2] 0 setdash\n"); break;    /* Long Dots */
   case 9 : fprintf(ips,"[6 10] 0 setdash\n"); break;   /* Short Dash */
   case 10: fprintf(ips,"[3 3 3 3 6 3] 0 setdash\n");
            break;                                      /* DotDot Dash */
   default: fprintf(ips,"[] 0 setdash\n"); break;
   }
}

/* paste on labels */
labelps(ips,xn,yn,angle,label)
FILE   *ips;
double xn,yn,angle;
char   *label;
{
   extern double scale;

   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeLbls scalefont setfont\n");
   fprintf(ips,"/label {");
   fprintf(ips,"newpath\n ");
   fprintf(ips,"%6.2f %6.2f moveto\n",-8*scale,-4*scale);
   fprintf(ips,"0     %6.2f rlineto\n",8*scale);
   fprintf(ips,"%6.2f 0     rlineto\n",16*scale);
   fprintf(ips,"0     %6.2f rlineto\n",-8*scale);
   fprintf(ips,"closepath\n 1.0 setgray \n fill \n 0 setgray\n");
   fprintf(ips,"0 0 moveto\n");
   fprintf(ips,"(%s) MIJL CEJ show } def\n",label);
   fprintf(ips,"%6.2f %6.2f translate\n",xn,yn);
   fprintf(ips,"%6.2f rotate\nlabel\n  %6.2f rotate\n",angle,-angle);
   fprintf(ips,"%6.2f %6.2f translate\n",-xn,-yn);
}

int set_label_pt(npts)
int npts;
{
   int rand();
   int i= -1,m;
   double mult;

   if (npts < 20)
      m = (int)(0.45*npts)-1;
   else {
      i = rand() % 10;
      mult = 0.1*i + 0.05;
      m = 10 + (int)(mult*(npts-20));
   }
   if (m==0) m=1;
   return(m);
}

/* End procedure */
endgrps(ips)
FILE *ips;
{
   fprintf(ips,"showpage\n");

   /* Close the file */
   fclose(ips);
}
